from __future__ import print_function
import os
import time
import sys
import numpy as np
import asap3
from asap3.mpi import world
from asap3.io.trajectory import _GetAtoms
from ase.io.bundletrajectory import BundleTrajectory as _BundleTrajectory
from ase.io.bundletrajectory import PickleBundleBackend
import ase.parallel
from asap3.io import cpu_setup
try:
    from ase.io.bundletrajectory import UlmBundleBackend
    _ulm_supported = True
except ImportError:
    _ulm_supported = False

    
class BundleTrajectory(_BundleTrajectory, _GetAtoms):
    """Reads and writes atoms into a .bundle directory.

    The BundleTrajectory is an alternative way of storing
    trajectories, intended for large-scale molecular dynamics
    simulations, where a single flat file becomes unwieldy.  Instead,
    the data is stored in directory, a 'bundle' (the name bundle is
    inspired from bundles in Mac OS, which are really just directories
    the user is supposed to think of as a single file-like unit).

    Parameters:

    filename:
        The name of the directory.  Preferably ending in .bundle.

    mode (optional):
        The file opening mode.  'r' means open for reading, 'w' for
        writing and 'a' for appending.  Default: 'r'.  If opening in
        write mode, and the filename already exists, the old file is
        renamed to .bak (any old .bak file is deleted), except if the
        existing file is empty.

    atoms (optional):
        The atoms that will be written.  Can only be specified in
        write or append mode.  If not specified, the atoms must be
        given as an argument to the .write() method instead.

    backup (optional):
        Use backup=False to disable renaming of an existing file.

    split (optional):
        If set to True or False, determines whether a split file
        format is used instead of the normal one.  In the split
        format, each processor in a parallel simulation writes its own
        files inside the BundleTrajectory, instead of leaving all I/O
        to the master.  In not specified, a split format is used if
        more than one million atoms.  Ignored in serial simulations.

    iolimit (optional):
        Limits the number of MPI tasks performing I/O simultaneously,
        to prevent overloading the NFS server.  Only enforced if the
        number of tasks is somewhat larger than the limit.
        
    backend=None:
        Request a backend.  Supported backends are 'pickle' and 'ulm'.
        Only honored when writing.  If not specified, the default is
        'ulm' if ASE supports it, otherwise it is 'pickle'.

    singleprecision=True:
        Store floating point data in single precision (ulm backend only).
        Note that for ASAP, the default is True!
    
    """
    def __init__(self, filename, mode='r', atoms=None,
                 backup=True, split=None, iolimit=10,
                 backend=None, singleprecision=True):
        if split is None:
            # Decide if subtype should be split based on number of atoms
            split = (atoms is not None) and atoms.get_number_of_atoms() > 1000000
        # Never use subtype split for serial simulations
        if not getattr(atoms, "parallel", False):
            split = False
        if split:
            self.subtype = 'split'
        else:
            self.subtype = 'normal'
        # self.iolimit may be needed if reading a split bundle.
        if world.size < 1.5 * iolimit:
            self.iolimit = None
        else:
            self.iolimit = iolimit

        if _ulm_supported:
            if backend is None:
                backend = 'ulm'
            _BundleTrajectory.__init__(self, filename, mode, atoms,
                                       backup=backup, backend=backend,
                                       singleprecision=singleprecision)
        else:
            _BundleTrajectory.__init__(self, filename, mode, atoms,
                                       backup=backup)
        if self.subtype == 'split':
            self.set_extra_data('ID')  # So the atoms can be sorted when read.
        
    def _set_defaults(self):
        subtype = self.subtype   # Preserve it
        _BundleTrajectory._set_defaults(self)
        self.subtype = subtype
        self.datatypes['forces'] = False

    def _open_write(self, atoms, *args, **kwargs):
        _BundleTrajectory._open_write(self, atoms, *args, **kwargs)
        # Use the collector object to join all data on master if subtype is normal
        # and the simulation is parallel.
        if self.subtype == 'normal' and atoms is not None and getattr(atoms, "parallel", False):
            self.atoms = asap3.Collector(atoms)

    def _open_append(self, atoms):
        _BundleTrajectory._open_append(self, atoms)
        if self.subtype == 'normal' and atoms is not None and getattr(atoms, "parallel", False):
            self.atoms = asap3.Collector(atoms)
        
    def _set_backend(self, backend=None):
        """Set the backed doing the actual I/O."""
        if backend is not None:
            self.backend_name = backend
        if self.backend_name == 'pickle':
            if self.subtype == 'normal':
                # Use the standard ASE backend
                self.backend = PickleBundleBackend(self.master)
            elif self.subtype == 'split':
                self.backend = PickleSplitBundleBackend(self.master,
                                                        self.iolimit)
        elif self.backend_name == 'ulm' and _ulm_supported:
            if self.subtype == 'normal':
                # Use the standard ASE backend
                self.backend = UlmBundleBackend(self.master, self.singleprecision)
            elif self.subtype == 'split':
                self.backend = UlmSplitBundleBackend(self.master,
                                                     self.iolimit,
                                                     self.singleprecision)            
        else:
            raise NotImplementedError(
                "This version of ASE cannot use BundleTrajectory with backend '%s'"
                % self.backend_name)

    def write(self, atoms=None):
        if self.subtype == 'normal' and atoms is not None and getattr(atoms, "parallel", False):
            atoms = asap3.Collector(atoms)
        _BundleTrajectory.write(self, atoms)

    def _make_bundledir(self, filename):
        """Make the main bundle directory.

        Since all MPI tasks might write to it, all tasks must wait for
        the directory to appear.

        For performance reasons, the first frame directory is created immediately.
        """
        assert not os.path.isdir(filename)
        world.barrier()
        if self.master:
            self.log("Making directory "+filename)
            os.mkdir(filename)
            framedir = os.path.join(filename, "F0")
            self.log("Making directory "+ framedir)    
            os.mkdir(framedir)
            world.barrier()
        else:
            world.barrier()
            i = 0
            while not os.path.isdir(filename):
                time.sleep(1)
                i += 1
            if i > 10:
                self.log("Waiting %d seconds for %s to appear!"
                         % (i, filename))

    def _make_framedir(self, frame):
        """Make subdirectory for the frame.

        For a split bundle, all MPI tasks write to the frame
        directory.  The slaves must therefore wait until it becomes
        available.  To minimize the waiting time, frames are
        pre-created.
        """
        if self.subtype == 'split':
            numdirs = 10
        else:
            numdirs = 1
        if self.master:
            for i in range(frame, frame+numdirs):
                framedir = os.path.join(self.filename, "F"+str(i))
                if not os.path.exists(framedir):
                    self.log("Making directory " + framedir)
                    os.mkdir(framedir)
        framedir = os.path.join(self.filename, "F"+str(frame))
        # Wait for the directory to appear
        world.barrier()
        i = 0
        while not os.path.isdir(framedir):
            time.sleep(1)
            i += 1
        if i > 10:
            self.log("Waiting %d seconds for %s to appear!"
                     % (i, framedir))
        return framedir

    def close(self):
        """Clean up when closing."""
        if self.state == 'write' and self.master:
            i = self.nframes
            while True:
                fname = os.path.join(self.filename, "F" + str(i))
                if not os.path.exists(fname):
                    break
                self.log("Closing, removing empty directory "+fname)
                os.rmdir(fname)
                i += 1
        _BundleTrajectory.close(self)

    def __del__(self):
        if self.state != 'closed':
            self.close()

    def get_atoms_distributed(self, n, cpus='auto', extra=None, verbose=True):
        """Return Atoms object number n from the trajectory.

        traj.get_atoms_distributed(n) reads the atoms from a
        BundleTrajectory written in split mode without needing to store
        everything in RAM on the master.  The master reads each file
        in the bundle, and transmits it to the slaves.  At the end,
        a distribution is made.

        Parameters:

        n: Which frame to read.  Use -1 for the last one.
        
        Optional parameters:
        cpus (default: 'auto'):  CPU layout to use, or the keyword 'auto'
            to figure it out automatically.
        
        extra:
            A list of names of extra arrays that should be read from the 
            input file.  The data is available with atoms.get_array.

        verbose (default: True):  Whether log messages should be written.
        """
        if verbose and (world.rank == 0 or verbose >= 2):
            if verbose >= 2:
                verbfile = open("get_atoms_distributed_{0}.log".format(world.rank), "w", 1)
            else:
                verbfile = sys.stdout
            def printlog(txt, *args):
                if world.rank == 0:
                    print("{}: {}".format(time.asctime(), txt), *args, file=verbfile)
        else:
            def printlog(txt, *args, **kwargs):
                pass
        if self.state != 'read':
            raise IOError('Cannot read in %s mode' % (self.state,))
        if n < 0:
            n += self.nframes
        if n < 0 or n >= self.nframes:
            raise IndexError('Trajectory index %d out of range [0, %d['
                             % (n, self.nframes))

        printlog("Beginning distributed read of {0} frame {1}".format(self.filename, n))
        if self.subtype != 'split':
            raise RuntimeError("Distributed read only possible for subtype='split'")
        if world.rank == 0:
            framedir = os.path.join(self.filename, 'F' + str(n))
            framezero = os.path.join(self.filename, 'F0')
            smalldata = self.backend.read_small(framedir)
            data = {}
            data['pbc'] = smalldata['pbc']
            assert data['pbc'].shape == (3,)
            data['cell'] = smalldata['cell'].astype(np.float64)
            assert data['cell'].shape == (3,3)
            data['constraint'] = smalldata['constraints']
            if data['constraint']:
                raise RuntimeError("Distributed read not possible: constraints are present.")
            natoms = np.zeros(1, np.int64)
            natoms[0] = smalldata['natoms']
            # The following data is NOT distributed
            nfragments = smalldata['fragments']
            smalldata0 = self.backend.read_small(framezero)
            nfragments0 = smalldata0['fragments']
        else:
            data = {}
            data['pbc'] = np.zeros((3,), dtype=np.bool)
            data['cell'] = np.zeros((3,3), dtype=np.float64)
            data['constraint'] = []
            natoms = np.zeros(1, np.int64)
        world.broadcast(data['pbc'], 0)
        world.broadcast(data['cell'], 0)
        world.broadcast(natoms, 0)
        natoms = natoms[0]
        # Which atoms should this process keep?
        begin_atom = world.rank * natoms // world.size
        end_atom = (world.rank + 1) * natoms // world.size
        # Create the empty atoms object
        atoms = ase.Atoms(**data)
        printlog("Beginning distribution of data.")
        printinterval = 100
        # Find which datatypes to send, and in which order.
        arraynames = ['positions', 'numbers', 'tags', 'masses', 'momenta']
        if extra is not None:
            arraynames.extend(extra)
        my_idarray = None
        for name in arraynames:
            if world.rank == 0:
                # The master decides if we send, and from which frame
                use = self.datatypes.get(name)
                if use == 'once':
                    nframes = np.array([nfragments0,], dtype=int)
                elif use:
                    nframes = np.array([nfragments,], dtype=int)
                else:
                    nframes = np.array([0,], dtype=int)
            else:
                nframes = np.zeros(1, dtype=int)
            world.broadcast(nframes, 0)
            nframes = nframes[0]
            if nframes:
                printlog("Transmitting {0} in {1} blocks".format(name, nframes))
                myarray = None  # No data yet.
                for frame in range(nframes):
                    if world.rank == 0:
                        # Read the data for later broadcast.
                        if use == 'once':
                            dirname = framezero
                        else:
                            dirname = framedir
                        ids = self.backend.read(dirname, 'ID_'+str(frame))
                        data = self.backend.read(dirname, name + '_' + str(frame))
                        info = {'dtype': str(data.dtype), 'dtype_id': str(ids.dtype),
                                'shape': data.shape}
                        ase.parallel.broadcast(info, 0, world)
                    else:
                        info = ase.parallel.broadcast(None, 0, world)
                        info['dtype_id'] = getattr(np, info['dtype_id'])
                        info['dtype'] = getattr(np, info['dtype'])
                        ids = np.zeros(info['shape'][0], dtype=info['dtype_id'])
                        data = np.zeros(info['shape'], dtype=info['dtype'])
                    if frame % printinterval == 0:
                        printlog("   block {0}, shape={1}, dtype={2}".format(frame,
                                                                             str(info['shape']),
                                                                             str(info['dtype'])))
                    world.broadcast(ids, 0)
                    world.broadcast(data, 0)
                    # Now all processes have this chunck of data.  Extract what we need.
                    if myarray is None:
                        myarrayshape = list(data.shape)
                        myarrayshape[0] = end_atom - begin_atom
                        myarray = np.zeros(tuple(myarrayshape), dtype=data.dtype)
                    if my_idarray is None:
                        # Create it as soon as we know know the dtype
                        my_idarray = np.arange(begin_atom, end_atom, dtype=ids.dtype)
                        atoms.arrays['ID'] = my_idarray
                        my_idarray_sanity = np.zeros(my_idarray.shape, dtype=my_idarray.dtype)
                    keep = np.logical_and(ids >= begin_atom, ids < end_atom)
                    ids = ids[keep]
                    idx = ids - begin_atom
                    assert (my_idarray[idx] == ids).all()
                    myarray[idx] = data[keep] # Assign the data to the relevant elements.
                    my_idarray_sanity[idx] = ids
                # We now have the whole array.
                # First check that we got all elements.
                assert (my_idarray == my_idarray_sanity).all()
                atoms.arrays[name] = myarray
            else:
                printlog("Not transmitting", name)
        printlog("All data distributed.")
        if cpus == 'auto':
            printlog("Calculating automatic CPU layout.")
            num_cores = world.size
            printlog("   num cpus:", num_cores)
            printlog("   cell:", np.diag(atoms.cell).tolist())
            cpus = cpu_setup.calculate_optimal_cpu_setup(num_cores, np.diag(atoms.cell).tolist())
            printlog("   cpu setup:", cpus)
            cpus = ase.parallel.broadcast(cpus, 0, world)
        printlog("Redistributing atoms according to positions.")
        atoms = asap3.MakeParallelAtoms(atoms, cpus)
        printlog("Redistribution done.")
        return atoms

    
if _ulm_supported:
    class UlmSplitBundleBackend(UlmBundleBackend):
        """A special backend for writing split bundles (ASAP only)."""
        def __init__(self, master, iolimit, singleprecision):
            # Store if this backend will actually write anything
            UlmBundleBackend.__init__(self, master, singleprecision)
            self.writesmall = master
            self.writelarge = True
            self.writenonarray = master
            self.iolimit = iolimit
            if iolimit:
                self.iostart = np.round(
                    np.linspace(0, world.size, iolimit+1)).astype(int)
                self.iotag = 797525
            self.lastwritedir = None

        def write_small(self, framedir, smalldata):
            "Write small data to be written jointly."
            smalldata['fragments'] = world.size
            UlmBundleBackend.write_small(self, framedir, smalldata)

        def write(self, framedir, name, data):
            "Write data to separate file."
            if hasattr(data, "shape"):
                # We need to store which kind of data was written in this frame 
                # so NFS synchronization is possible when closing file.
                if framedir != self.lastwritedir:
                    self.lastwritedir = framedir
                    self.writenames = []
                self.writenames.append(name)
                # As expected, we are writing a NumPy array
                self.iosync_start()
                name = "%s_%d" % (name, world.rank)
                UlmBundleBackend.write(self, framedir, name, data)
                self.iosync_end()
            elif self.writenonarray:
                # If the data is not a NumPy array, only the master writes.
                UlmBundleBackend.write(self, framedir, name, data)

        def read(self, framedir, name):
            "Read data from separate file."
            self.iosync_start()
            x = UlmBundleBackend.read(self, framedir, name)
            self.iosync_end()
            return x

        def iosync_start(self):
            "Prevents too many simultaneous IO tasks from trashing server."
            if self.iolimit and world.rank not in self.iostart:
                # I must wait.
                token = np.zeros(1, int)
                world.receive(token, world.rank-1, self.iotag)

        def iosync_end(self):
            if self.iolimit and world.rank+1 not in self.iostart:
                # Another task is waiting for me.
                token = np.zeros(1, int)
                world.send(token, world.rank+1, self.iotag)

        def close(self, log=None):
            """Make sure that all data is available on disk for all MPI tasks."""
            if self.lastwritedir:
                for name in self.writenames:
                    for part in range(world.size):
                        fname = os.path.join(self.lastwritedir, "%s_%d.ulm" % (name, part))
                        if not os.path.exists(fname):
                            if log:
                                log.write("Task %i is waiting for '%s' to appear.\n" %
                                          (world.rank, fname))
                            for i in range(20):
                                time.sleep(5)
                                if os.path.exists(fname):
                                    break
                            if not os.path.exists(fname) and log:
                                log.write("WARNING: Task %i gave up waiting for '%s'.\n" %
                                          (world.rank, fname))
            self.lastwritedir = None  # Do not repeat the above.
        
class PickleSplitBundleBackend(PickleBundleBackend):
    """A special backend for writing split bundles (ASAP only)."""
    def __init__(self, master, iolimit):
        # Store if this backend will actually write anything
        self.writesmall = master
        self.writelarge = True
        self.writenonarray = master
        self.iolimit = iolimit
        if iolimit:
            self.iostart = np.round(
                np.linspace(0, world.size, iolimit+1)).astype(int)
            self.iotag = 413151
        self.lastwritedir = None
            
    def write_small(self, framedir, smalldata):
        "Write small data to be written jointly."
        smalldata['fragments'] = world.size
        PickleBundleBackend.write_small(self, framedir, smalldata)
        
    def write(self, framedir, name, data):
        "Write data to separate file."
        if hasattr(data, "shape"):
            # We need to store which kind of data was written in this frame 
            # so NFS synchronization is possible when closing file.
            if framedir != self.lastwritedir:
                self.lastwritedir = framedir
                self.writenames = []
            self.writenames.append(name)
            # As expected, we are writing a NumPy array
            self.iosync_start()
            name = "%s_%d" % (name, world.rank)
            PickleBundleBackend.write(self, framedir, name, data)
            self.iosync_end()
        elif self.writenonarray:
            # If the data is not a NumPy array, only the master writes.
            PickleBundleBackend.write(self, framedir, name, data)

    def read(self, framedir, name):
        "Read data from separate file."
        self.iosync_start()
        x = PickleBundleBackend.read(self, framedir, name)
        self.iosync_end()
        return x
    
    def iosync_start(self):
        "Prevents too many simultaneous IO tasks from trashing server."
        if self.iolimit and world.rank not in self.iostart:
            # I must wait.
            token = np.zeros(1, int)
            world.receive(token, world.rank-1, self.iotag)

    def iosync_end(self):
        if self.iolimit and world.rank+1 not in self.iostart:
            # Another task is waiting for me.
            token = np.zeros(1, int)
            world.send(token, world.rank+1, self.iotag)
            
    def close(self, log=None):
        """Make sure that all data is available on disk for all MPI tasks."""
        if self.lastwritedir:
            for name in self.writenames:
                for part in range(world.size):
                    fname = os.path.join(self.lastwritedir, "%s_%d.pickle" % (name, part))
                    if not os.path.exists(fname):
                        if log:
                            log.write("Task %i is waiting for '%s' to appear.\n" %
                                      (world.rank, fname))
                        for i in range(20):
                            time.sleep(5)
                            if os.path.exists(fname):
                                break
                        if not os.path.exists(fname) and log:
                            log.write("WARNING: Task %i gave up waiting for '%s'.\n" %
                                      (world.rank, fname))
        self.lastwritedir = None  # Do not repeat the above.
        




from __future__ import print_function
from asap3 import CoordinationNumbers
import numpy as np
from asap3.MonteCarlo.Moves import SurfaceMove
from ase import Atom
import random as rd
#1.1 Now removes 1 atom at a time.
def mychoice(a, n):
	#Could be obsolete now.
	"Replaces numpy.random.choice(a, n, False) as our numpy is ancient."
	a = list(a)
	while len(a) > n:
		# Remove a random element
		del a[np.random.randint(len(a))]
	return np.array(a)

def resizecluster(atoms, nwanted):
	print("Resizing Cluster 1 atom at a time")
	nactual = len(atoms)
	if nactual == nwanted:
		return
	elif nactual > nwanted:
		removeatoms(atoms, nactual - nwanted)		
	else:
		addatoms(atoms, nwanted - nactual)
		
def removeatoms(atoms, n):
	"Remove n atoms from the cluster."
	for i in range(n):
		# We still have atoms to remove
		# Find the ones with lowest coordination number
		coords = CoordinationNumbers(atoms)
		coordination = coords.min()
		idx = np.arange(len(atoms))
		candidates = idx[np.equal(coords, coordination)]
		# candidates now contains the indices of the atoms with
		# low coordination number
		atom_delete = np.random.randint(len(candidates))
		del atoms[candidates[atom_delete]]

def addatoms(atoms,n):
	element = atoms[0].symbol
	SM = SurfaceMove()
	
	for i in range(n):
		SM.set_atoms(atoms)
		idx = SM.vacant_indexes()
		#Find the n highest coordinated sites(this is in ids of SM)
		coords = SM.vacant_coordinations[idx]
		candidates = idx[np.equal(coords,coords.max())]
		vacPos = SM.vacant_positions
		chosenID = np.random.randint(len(candidates)) #Random ids
		#print "Adding atom at site", id1
		atoms.append(Atom(element, position=vacPos[chosenID]))
		
		
def objarraysize(arr): #Outputs the size of array objects of arrays
	out = 0
	for i in range(0,arr.size):
		out+=arr[i].size
	return out



"""Handles theads.

In particular checks if you are allowed to use them on a cluster.

This module also disables OpenMP threads initially unless the user
has requested them.
"""
from __future__ import print_function
import numpy 
import sys
import os
from asap3.Internal.Builtins import _asap

verbose = True

def AsapThreads(n = None, force=False, log=None):
    """Activates the use of threads in calculators that support it.

    The optional argument n can be used to specify the number of
    threads to use.  If not given, it will be set to the number of
    CPUs.

    The optional argument force can be set to true to override the
    security mechanisms on machines where it fails.  IT IS STRICTLY
    FORBIDDEN TO USE THE force PARAMETER ON NIFLHEIM!

    Return value: The number of threads
    """
    if not _asap.support_openmp():
        if log is not None: 
            log.write("OpenMP not supported\n")
        raise RuntimeError("AsapThreads: OpenMP threads not supported.")
    if log is not None: 
        log.write("OpenMP supported\n")
    ncpu = _asap.get_num_procs()
    noriginal = n
    if log is not None: 
        log.write("Found {0} cpus\n".format(ncpu))
    if n is None:
        if ncpu is not None:
            if verbose:
                print("AsapThreads: Choosing", ncpu, "threads.")
            n = ncpu
        else:
            raise ValueError("Cannot determine number of processors.  You must specify the number of threads.")

    if n == 1:
        if noriginal == None:
            print("AsapThreads warning: Only one CPU found, disabling threads.", file=sys.stderr)
        _asap.set_num_threads(1)
        return 1

    if force:
        if log is not None: 
            log.write("Forcing {0} threads\n".format(n))
        print("AsapThreads warning: force specified, hope you know what you are doing.  Enabling", n, "threads.", file=sys.stderr)
        _asap.set_num_threads(n)
        return n

    if n > ncpu:
        raise ValueError("Cannot start %d threads: only %d CPUs" % (n, ncpu))

    Check_MPI_Consistency(n, ncpu)
    Check_PBS_Allowance(n)
    
    _asap.set_num_threads(n)
    return n


def Check_MPI_Consistency(n, ncpu):
    """Check that we run the same number of threads on all nodes."""
    try:
        from asap3.mpi import world
        worldsize = world.size
        if verbose and worldsize > 1:
            print("MPI world size is", worldsize, file=sys.stderr)
    except (ImportError, AttributeError):
        return   # No MPI support
    if worldsize == 1:
        return
    data = numpy.array([n, ncpu])
    world.broadcast(data, 0)
    if n != data[0] or ncpu != data[1]:
        raise RuntimeError("AsapThreads: Inconsitent CPU layout across the MPI processor pool.")

def Check_PBS_Allowance(n):
    "Check that we are not running more threads than allowed by the PBS batch system."
    try:
        from asap3.mpi import world
        worldsize = world.size
    except (ImportError, AttributeError):
        worldsize = 1
        
    try:
        nodefile = os.environ["PBS_NODEFILE"]
        nodelist = open(nodefile).readlines()   
    except (KeyError, IOError):
        return   # No PBS or not on master.

    nodes = {}
    for node in nodelist:
        try:
            nodes[node] += 1
        except KeyError:
            nodes[node] = 1
    
    if len(nodes) != worldsize:
        print("MPI world size:", worldsize, file=sys.stderr)
        print("Number of distinct nodes:", len(nodes), file=sys.stderr)
        for node in nodelist:
            print(sys.stderr, "   ", node)
        raise RuntimeError("Number of MPI nodes is different from number of distinct nodes in PBS_NODEFILE.")
    for x in nodes.keys():
        if nodes[x] < n:
            raise RuntimeError("Attempting to run %d threads but %d jobs allowed on node %s" % (n, nodes[x], x))

# When this module is loaded as part of ASAP startup, it should
# disable OpenMP threads, unless the user has specified the
# OMP_NUM_THREADS environment variable.  Also disables processor
# affinity if compiled with a compiler that enables it per default
# (such as Open64 with OpenMP enabled), as affinity will force
# multiple MPI tasks to run on the same cpu while other cpus are
# empty.

if "OMP_NUM_THREADS" not in os.environ or os.environ["OMP_NUM_THREADS"] == "1":
    # We should turn of OpenMP threads until the user eventually enables them.
    if "O64_OMP_SET_AFFINITY" in os.environ:
        # The user controls affinity
        _asap.set_num_threads(1)
    else:
        # Also disable affinity.
        _asap.set_num_threads(1, noaffinity=True)
else:
    print("asap3...Threads: Leaving threads enabled as OMP_NUMTHREADS=%s" % (os.environ["OMP_NUM_THREADS"]))



"""Locate the position of a cluster (its center of mass) in a simulation.

The module calculates the center of mass of a connected cluster of atoms,
even in the case where the cluster straddles the periodic boundaries.  It 
is OK that there are other atoms not connected to the cluster, as long
as the atom with the highest coordination number is part of the cluster,
or an atom being part of the cluster is specified manually.
"""
from __future__ import print_function

from asap3 import FullNeighborList
import numpy as np
import sys
from ase.io import Trajectory, BundleTrajectory
from ase.utils import basestring

class ClusterCenter:
    def __init__(self, cutoff=3.0):
        self.nblist = FullNeighborList(cutoff)
        
    def calculate_center(self, atoms, startatom=None):
        """Calculate the center of mass position of a cluster of atoms.
        
        An atom belonging to the cluster can optionally be specified, if
        not specified one of the highest coordinated atoms is used.
        """
        natoms = len(atoms)
        self.nblist.check_and_update(atoms)
        if startatom is None:
            coordnum = [len(self.nblist[i]) for i in range(natoms)]
            startatom = np.argmax(coordnum)
        self.cluster = []
        self.sumpos = np.zeros(3)
        self.summass = 0
        self.isincluster = np.zeros(natoms, bool)
        self.masses = atoms.get_masses()
        self.add_atom_to_cluster(startatom, atoms[startatom].position)
        com = self.sumpos / self.summass
        return com
    
    def add_atom_to_cluster(self, n, pos):
        "Add an atom and all its neighbors to the cluster."
        self.isincluster[n] = True
        self.sumpos += pos * self.masses[n]
        self.summass += self.masses[n]
        neighbors, reldists, sqdists = self.nblist.get_neighbors(n)
        for i, relpos in zip(neighbors, reldists):
            if not self.isincluster[i]:
                # Add neighboring atom to cluster, using the relative
                # position so periodic boundaries are handled correctly.
                self.add_atom_to_cluster(i, pos + relpos)
            
    def calculate_from_trajectory(self, traj, startatom=None, selector=None):
        """Calculate the center of mass for a cluster in a trajectory file.
        
        traj: The trajectory object, or a file name.
        
        startatom (optional): Specifies an atom guaranteed to be in the cluster.
            If not specified, the atom with the highest coordination number is
            used (if there is only one cluster this should work).
            
        selector (optional): A function defining which atoms should be 
            considered.  The function is called with one argument, the atoms
            object, and should either return an array of booleans, one per
            atom, indicating if they should be included, or return an array
            of integers interpreted as the indices of the atoms to be included.
            This can e.g. be used to select a cluster sitting on a substrate.
            
        This method returns an array of center-of-mass positions, one for each
        frame in the trajectory.
        """
        if isinstance(traj, basestring):
            if traj.endswith('.traj'):
                traj = Trajectory(traj)
            elif traj.endswith('.bundle'):
                traj = BundleTrajectory(traj)
            else:
                raise ValueError("Cannot handle a file name not ending in .traj or .bundle: " + traj)
        result = []
        for atoms in traj:
            if selector is not None:
                idx = selector(atoms)
                atoms = atoms[idx]
            result.append(self.calculate_center(atoms, startatom))
        return result
    
if __name__ == '__main__':
    # Demo example: Track a cluster of Ni atoms on a surface of something else.
    def sel(a):
        return np.equal(a.get_atomic_numbers(), 28)
    res = ClusterCenter().calculate_from_trajectory(sys.argv[1], selector=sel)
    print(np.array(res))
    



# VelocityDistributions.py -- set up a velocity distribution

"""Module for setting up e.g. Maxwell-Boltzmann velocity distributions.

Currently, only one function is defined, MaxwellBoltzmannDistribution,
which sets the momenta of a list of atoms according to a
Maxwell-Boltzmann distribution at a given temperature.
"""

import numpy as np
import asap3.mpi
from ase.md.velocitydistribution import _maxwellboltzmanndistribution

class _fakecomm:
    "Do-nothing communicator for GPAW-like communication."
    def broadcast(self, data, root):
        pass


def MaxwellBoltzmannDistribution(atoms, temp, force_temp=False):
    """Sets the momenta to a Maxwell-Boltzmann distribution. temp should be
    fed in energy units; i.e., for 300 K use temp=300.*units.kB. If
    force_temp is set to True, it scales the random momenta such that the
    temperature request is precise.
    """
    momenta = _maxwellboltzmanndistribution(atoms.get_masses(), temp,
                                            _fakecomm())
    atoms.set_momenta(momenta)
    if force_temp:
        temp0 = atoms.get_kinetic_energy() / atoms.get_number_of_atoms() / 1.5
        gamma = temp / temp0
        atoms.set_momenta(atoms.get_momenta() * np.sqrt(gamma))


def Stationary(atoms):
    "Sets the center-of-mass momentum to zero."
    p = atoms.get_momenta()
    p0 = np.sum(p, 0)
    # We should add a constant velocity, not momentum, to the atoms
    m = atoms.get_masses()
    mtot = np.sum(m)
    if getattr(atoms, "parallel", False):
        data = np.zeros(4, float)
        data[:3] = p0
        data[3] = mtot
        asap3.mpi.world.sum(data)
        p0 = data[:3]
        mtot = data[3]
    v0 = p0/mtot
    p -= v0*m[:,np.newaxis]
    atoms.set_momenta(p)
    



from ase import Atom, Atoms
import numpy as np
import numbers

_mc_err = "Cannot access the position(s) and atomic_number(s) attribute of MC optimized atoms."

class MC_Atom(Atom):
    """A single atom in a Monte Carlo optimized Atoms object."""
    #__slots__ = Atom.__slots__
    
    def get_position(self): 
        p = self.get('position')
        p.flags.writeable = False
        return p

    def set_position(self, position):
        Atom.set(self, 'position', position)
        self._mark()

    def get_atomic_number(self):
        return self.get('number')
    
    def set_atomic_number(self, number):
        Atom.set(self, 'number', number)
        self._mark()
        
    def _mark(self):
        if self.atoms is not None:
            mco = self.atoms.mc_optim
            mco[0] = n = mco[0] + 1
            if n < 100:
                mco[n] = self.index

    def _oops(self):
        raise NotImplementedError(_mc_err)

    number = property(get_atomic_number, set_atomic_number, doc='The atomic number')
    position = property(get_position, set_position, doc='The position (gettting it this way is a read-only array')    

class MonteCarloAtoms(Atoms):
    """A list of atoms class optimized for Monte Carlo simulations.

    This is intended to be used together with the MonteCarloEMT calculator.
    """
    def __init__(self, *args, **kwargs):
        self.mc_optim = np.zeros(101, np.intc)
        Atoms.__init__(self, *args, **kwargs)
        
    def __getitem__(self, i):
        if isinstance(i, numbers.Integral):
            natoms = len(self)
            if i < -natoms or i >= natoms:
                raise IndexError('Index out of range.')

            return MC_Atom(atoms=self, index=i)
        else:
            return Atoms.__getitem__(self, i)
    
    def set_calculator(self, calc=None):
        """Attach calculator object."""
        Atoms.set_calculator(self, calc)
        self.mc_optim[0] = 10000  # MC optim invalid

    def get_potential_energy(self):
        epot = Atoms.get_potential_energy(self)
        self.mc_optim[0] = 0
        return epot

    def get_potential_energies(self):
        epot = Atoms.get_potential_energies(self)
        self.mc_optim[0] = 0
        return epot

    def set_positions(self, newpositions):
        """Set positions."""
        Atoms.set_positions(self, newpositions)
        self.mc_optim[0] = 10000  # MC optim invalid

    def get_test(self):
        return True
        



from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import *
from numpy import *
import ase.data

def TestLists(nblist, nblist2, name, count=None):
    "Run the tests on a half and a full neighbor list."
    print("Running tests on", name)
    print("Testing %s: Lengths of lists." % (name,))
    if count:
        sum = 0
        for lst in nblist:
            sum += len(lst)
        ReportTest("Absolute length of list", sum, count*len(atoms), 0)

    sum1 = sum2 = 0
    for nb in enumerate(nblist):
        sum1 += len(nb)
    for nb in enumerate(nblist2):
        sum2 += len(nb)
        
    ReportTest("Equal total length of lists", sum2, sum1, 0)
    
    print("Testing %s: List 1 atoms on list 2." % (name,))
    for i, nb in enumerate(nblist):
        for jj in nb:
            j = int(jj)
            ReportTest.BoolTest("Atom %d on list %d (forward)" % (j, i),
                                (j in nblist2[i]) != (i in nblist2[j]),
                                silent=True)
        if ReportTest.GetNumberOfErrors() > 10:
            print("*** Too many errors - giving up! ***")
            break
    
    print("Testing %s: List 2 atoms on list 1." % (name,))
    for i, nb in enumerate(nblist2):
        for jj in nb:
            j = int(jj)
            ReportTest.BoolTest("Atom %d on list %d (reverse)" % (j, i),
                                (j in nblist[i]) != (i in nblist[j]),
                                silent=True)
        if ReportTest.GetNumberOfErrors() > 10:
            print("*** Too many errors - giving up! ***")
            break
    

print_version(1)

element = "Cu"
latconst = ase.data.reference_states[ase.data.atomic_numbers[element]]['a']

atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,1],[0,0,1]], size=(9,7,5),
                          symbol=element, debug=0, pbc=(0,0,0))
print("Symmetry:", atoms.get_pbc())
atoms.set_calculator(EMT())
epot = atoms.get_potential_energy()

nblist1 = NeighborList(latconst * 0.5 * (1/sqrt(2) + 1), atoms, 0.0)
nblist2 = NeighborCellLocator(latconst * 0.5 * (1/sqrt(2) + 1), Atoms(atoms), 0.0)
TestLists(nblist1, nblist2, "nearest-neigbor lists (free)")

atoms = Atoms(atoms, pbc=(1,1,1))
print("Symmetry:", atoms.get_pbc())

nblist1 = NeighborList(latconst * 0.5 * (1/sqrt(2) + 1), atoms, 0.0)
nblist2 = NeighborCellLocator(latconst * 0.5 * (1/sqrt(2) + 1), Atoms(atoms), 0.0)
TestLists(nblist1, nblist2, "nearest-neigbor lists (periodic)", 6)

nblist1 = NeighborCellLocator(4.98409, atoms, 0.0)
nblist2 = NeighborList(4.98409, Atoms(atoms), 0.0)
TestLists(nblist1, nblist2, "long neigbor lists (periodic)", 21)

atoms = Atoms(atoms, pbc=(0,0,0))
print("Symmetry:", atoms.get_pbc())

nblist = NeighborCellLocator(4.98409, atoms, 0.0)
nblist2 = NeighborList(4.98409, Atoms(atoms), 0.0)
TestLists(nblist, nblist2, "long neigbor lists (free)")

atoms1 = Atoms(atoms, pbc=(1,1,1))
atoms2 = Atoms(atoms, pbc=(1,1,1))
nblist = NeighborCellLocator(4.98409, atoms1)
nblist2 = NeighborList(4.98409, atoms2)

x0 = atoms1.get_positions()[2,0]
for i in range(30):
    r = atoms1.get_positions()
    r[2,0] += 0.05
    atoms1.set_positions(r)
    print(r[2,0], end=' ')
    r = atoms2.get_positions()
    r[2,0] += 0.05
    print(r[2,0])
    atoms2.set_positions(r)
    nblist.check_and_update(atoms1)
    nblist2.check_and_update(atoms2)
    TestLists(nblist, nblist2, "Translation step "+str(i))

ReportTest.Summary()



"""Test the FixAtoms constraint and the Subset filter."""
from __future__ import print_function

from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution
from asap3.md.verlet import VelocityVerlet
from ase.constraints import Filter
from asap3.constraints import FixAtoms
from ase.constraints import FixAtoms as ASE_FixAtoms
from asap3.testtools import ReportTest
import numpy as np

init = FaceCenteredCubic(size=(10,10,10), symbol='Cu', pbc=False)
z = init.get_positions()[:,2]
fixedatoms = np.less(z, 0.501*z.max())
print(len(init), sum(fixedatoms))
MaxwellBoltzmannDistribution(init, 2000*units.kB)

print()
print("Running simulation with Filter")
atoms1 = Atoms(init)
atoms1.set_calculator(EMT())
atoms1a = Filter(atoms1, mask=np.logical_not(fixedatoms))

dyn = VelocityVerlet(atoms1a, 0.5*units.fs)
dyn.run(50)
r1 = atoms1.get_positions()

print()
print("Running simulation with Asap's FixAtoms")
atoms2 = Atoms(init)
atoms2.set_calculator(EMT())
atoms2.set_constraint(FixAtoms(mask=fixedatoms))

dyn = VelocityVerlet(atoms2, 0.5*units.fs)
dyn.run(50)
r2 = atoms2.get_positions()

print()
print("Running simulation with ASE's FixAtoms")
atoms3 = Atoms(init)
atoms3.set_calculator(EMT())
# Just to be difficult, convert mask to indices
indx = np.compress(fixedatoms, np.arange(len(atoms3)))
assert len(indx) == fixedatoms.sum()
atoms3.set_constraint(ASE_FixAtoms(indx))

dyn = VelocityVerlet(atoms3, 0.5*units.fs)
dyn.run(50)
r3 = atoms2.get_positions()

err = np.max(np.abs(r1 - r2).flat)
print()
print("Filter and Asap's FixAtoms:", err)
ReportTest("Identical positions (Filter and Asap's FixAtoms)", err, 0.0, 1e-9)

err = np.max(np.abs(r2 - r3).flat)
print("ASE's and Asap's FixAtoms:", err)
ReportTest("Identical positions (ASE's and Asap's FixAtoms)", err, 0.0, 1e-9)

print()
print("Running Langevin simulation with Asap's FixAtoms")
atoms4 = Atoms(init)
atoms4.set_calculator(EMT())
atoms4.set_constraint(FixAtoms(mask=fixedatoms))

dyn = Langevin(atoms4, 0.5*units.fs, 1000*units.kB, 0.01)
dyn.run(50)

print()
print("Running Langevin simulation with ASE's FixAtoms")
atoms5 = Atoms(init)
atoms5.set_calculator(EMT())
atoms5.set_constraint(ASE_FixAtoms(mask=fixedatoms))

dyn = Langevin(atoms5, 0.5*units.fs, 1000*units.kB, 0.01)
dyn.run(50)

print()
sanity = [[atoms1, "Verlet + Filter"],
          [atoms2, "Verlet + Asap's FixAtoms"],
          [atoms3, "Verlet + ASE's FixAtoms"],
          [atoms4, "Langevin + Asap's FixAtoms"],
          [atoms5, "Langevin + ASE's FixAtoms"],
          ]
r_init = init.get_positions()
for a, label in sanity:
    print("Sanity check, %s:" % (label,))
    ok = (a.get_positions() == r_init) + np.logical_not(fixedatoms)[:,np.newaxis]
    if ok.all():
        print("  Stationary atoms have not moved: OK")
    else:
        raise RuntimeError("Stationary atoms have moved (%s)" % (label,))
    ok = (a.get_positions() != r_init) + fixedatoms[:,np.newaxis]
    if ok.all():
        print("  Mobile atoms have moved: OK")
    else:
        raise RuntimeError("Mobile atoms have not moved (%s)" % (label,))
    

ReportTest.Summary()







from ase.md.npt import NPT as aseNPT
from asap3.md.md import ParallelMolDynMixin
from asap3.mpi import world
from ase.utils import basestring
import sys

if hasattr(aseNPT, '_npt_version'):
    class NPT(aseNPT, ParallelMolDynMixin, object):        
        def __init__(self, atoms, 
                     timestep, temperature, externalstress, ttime, pfactor,
                     mask=None, trajectory=None, logfile=None, loginterval=1):
            ParallelMolDynMixin.__init__(self, 'NPT', atoms)
            aseNPT.__init__(self, atoms, 
                     timestep, temperature, externalstress, ttime, pfactor,
                     mask=mask, trajectory=trajectory, logfile=logfile,
                     loginterval=loginterval)

        def _getnatoms(self):
            """Get the number of atoms.

            In a parallel simulation, this is the total number of atoms on all
            processors.
            """
            return self.atoms.get_number_of_atoms()    

        def get_center_of_mass_momentum(self):
            "Get the center of mass momentum."
            cm = self.atoms.get_momenta().sum(0)
            world.sum(cm)
            return cm

        @classmethod
        def read_from_trajectory(cls, trajectory,  frame=-1, cpus=None, extra=None):
            """Read dynamics and atoms from trajectory (Class method).

            Simultaneously reads the atoms and the dynamics from a BundleTrajectory,
            including the internal data of the NPT dynamics object (automatically
            saved when attaching a BundleTrajectory to an NPT object).

            Arguments::

            trajectory 
                The filename or an open asap3.io.BundleTrajectory object.

            frame (optional)
                Which frame to read.  Default: the last. 

            cpus (optional)
                If specified, the atoms are read in parallel and distributed
                for a parallel simulation.  This argument should then be the
                cpu layout (a tuple of three integers).
            """
            if isinstance(trajectory, basestring):
                if trajectory.endswith('/'):
                    trajectory = trajectory[:-1]
                if trajectory.endswith('.bundle'):
                    from asap3.io.bundletrajectory import BundleTrajectory
                    trajectory = BundleTrajectory(trajectory)
                else:
                    raise ValueError("Cannot open '%': unsupported file format" % trajectory)
            atoms = trajectory.get_atoms(frame, cpus, extra=extra)
            # The original read_from_trajectory method does the rest of the
            # work.  As it is a class method, the unbound method is not available
            # in the usual way, but as the im_func attribute.  Change to __func__
            # in Python 3.X
            return aseNPT.read_from_trajectory.__func__(cls, trajectory, 
                                                       frame=frame, atoms=atoms)   

        # Store these arrays on the atoms
        q = property(lambda s: s.get("q"), lambda s, x: s.set("q", x))
        q_past = property(lambda s: s.get("q_past"), lambda s, x: s.set("q_past", x))
        q_future = property(lambda s: s.get("q_future"), lambda s, x: s.set("q_future", x))

else:  
    # hasattr(aseNPT, '_npt_version'):
    sys.stderr.write("Warning: Asap.NPT based on old ASE version.")
    class NPT_OldAse(aseNPT, object):
        def __init__(self, atoms, *args, **kwargs):
            aseNPT.__init__(self, atoms, *args, **kwargs)
            if getattr(atoms, "parallel", False):
                raise NotImplementedError("NPT dynamics does not work in parallel with this version of ASE.  Please use the developer version.")
    NPT = NPT_OldAse




from __future__ import print_function
from asap3 import *
import pickle
from numpy import *
import sys, os, time
from asap3.testtools import *
from asap3.mpi import world
from ase.lattice.cubic import *

debug = 0
if debug == 1:
    DebugOutput("moveatomsLJ%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

print_version(1)
#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]
    
picklefile = "moveatomsLJ.pickle"

if isparallel:
    print("RUNNING PARALLEL VERSION OF TEST SCRIPT")
else:
    print("RUNNING SERIAL VERSION OF TEST SCRIPT")

if ismaster:
    atoms = FaceCenteredCubic(size=(10,10,10), symbol="Cu",
                              latticeconstant=1.09*2.7*1.41,
                              pbc=True)
else:
    atoms = None

#set_verbose(1)

if isparallel:
    atoms = MakeParallelAtoms(atoms, cpulayout, distribute=False)
    atoms.ghosts["ID"] = zeros(0,int)
    print("Distributing")
    atoms.distribute()
    nTotalAtoms = atoms.get_number_of_atoms()
else:
    nTotalAtoms = len(atoms)
    atoms.arrays["ID"] = arange(nTotalAtoms)

print("Setting potential")
atoms.set_calculator(LennardJones([29], [0.15], [2.7], 2.7*1.9, True))

try:
    expected = pickle.load(open(picklefile, "rb"))
except IOError:
    expected = None
    newexpected = []


def report():
    for i in range(world.size):
        if i == world.rank:
            print("Data on processor", i)
            for key in atoms.arrays.keys():
                print("  ", key, atoms.arrays[key].shape)
            r = atoms.get_positions()
            if len(r):
                print("Limits to the positions:")
                print ("[%.4f, %.4f]  [%.4f, %.4f]  [%.4f, %.4f]" %
                       (min(r[:,0]), max(r[:,0]), min(r[:,1]), max(r[:,1]),
                        min(r[:,2]), max(r[:,2])))
            if world.size > 1:
                print("Ghost data on processor", i)
                for key in atoms.ghosts.keys():
                    print("  ", key, atoms.ghosts[key].shape)
                r = atoms.ghosts['positions']
                if len(r):
                    print("Limits to the ghost positions:")
                    print ("[%.4f, %.4f]  [%.4f, %.4f]  [%.4f, %.4f]" %
                           (min(r[:,0]), max(r[:,0]), min(r[:,1]), max(r[:,1]),
                            min(r[:,2]), max(r[:,2])))
        world.barrier()


for i in range(10):
    print("Step number", i)
    if i:
        dz = 0.1 * sin(atoms.arrays["ID"] / 10.0)
        r = atoms.get_positions()
        r[:,2] += dz
        atoms.set_positions(r)
    e = atoms.get_potential_energy() / nTotalAtoms
    print("\n%8f" % (e,))
    if expected:
        ReportTest("Step "+str(i), e, expected[i], 1e-9)
    else:
        newexpected.append(e)
    #report()
    #print atoms.get_potential_energies()
    
if expected is None and ismaster:
    print("Saving energies")
    out = open(picklefile, "wb")
    pickle.dump(newexpected, out)
    out.close()
    
ReportTest.Summary()




from __future__ import print_function
from asap3 import *
from ase.md.verlet import VelocityVerlet
from ase.lattice.cubic import FaceCenteredCubic
from asap3.io.trajectory import *
from numpy import *
from asap3.mpi import world
import sys, os, time
from asap3.testtools import ReportTest

debug = 0
if debug == 1:
    DebugOutput("parallelLJ%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)
    
#DebugOutput("output.%d")
print_version(1)

worldsize = world.size
ismaster = world.rank == 0

if worldsize == 1:
    layout = [None]
elif worldsize == 2:
    layout = [(2,1,1), (1,2,1), (1,1,2)]
elif worldsize == 3:
    layout = [(3,1,1), (1,1,3)]
elif worldsize == 4:
    layout = [(1,2,2), (2,1,2), (2,2,1)]
elif worldsize == 8:
    layout = [(2,2,2), (1,2,4)]
else:
    raise ValueError("Cannot run on %d CPUs." % (worldsize,))

elements = [29]
epsilon  = [0.15]
sigma    = [2.7]


if ismaster:
    initial = FaceCenteredCubic(directions=((1,0,0),(0,1,0),(0,0,1)),
                                size=(40,40,40), symbol="Cu",
                                latticeconstant=1.09*sigma[0]*1.41,
                                pbc=(1,1,0))
    momenta = sqrt(2*63.5 * units.kB * 400) * sin(arange(3*len(initial)))
    momenta.shape = (-1,3)
    initial.set_momenta(momenta)
    stdout = sys.stdout
    print("Number of atoms:", len(initial))
else:
    initial = None
    stdout = open("/dev/null", "w")

for cpulayout in layout:
    if cpulayout:
        print("Test with layout "+str(cpulayout), file=stdout)
        atoms = MakeParallelAtoms(initial, cpulayout)
        natoms = atoms.get_number_of_atoms()
    else:
        print("Serial test", file=stdout)
        atoms = Atoms(initial)
        natoms = len(atoms)
    print("Number of atoms:", natoms)
    temp = atoms.get_kinetic_energy() / (1.5*units.kB*natoms)
    print("Temp:", temp, "K", file=stdout)
    ReportTest("Initial temperature", temp, 400.0, 1.0)
    atoms.set_calculator(LennardJones(elements, epsilon, sigma, -1.0, True))

    epot = atoms.get_potential_energy()
    print("Potential energy:", epot, file=stdout)
    ReportTest("Initial potential energy", epot, -301358.3, 0.5)
    etot = epot + atoms.get_kinetic_energy()


    dyn = VelocityVerlet(atoms, 3*units.fs)
    if 0:
        traj = Trajectory("llj.traj", "w", atoms)
        dyn.attach(traj, interval=2)
                
    etot2 = None
    for i in range(5):
        dyn.run(15)
        newetot = atoms.get_potential_energy()+ atoms.get_kinetic_energy()
        print("Total energy:", newetot, file=stdout)
        temp = atoms.get_kinetic_energy() / (1.5*units.kB*natoms)
        print("Temp:", temp, "K", file=stdout)
        if etot2 == None:
            ReportTest("Total energy (first step)", newetot, etot, 40.0)
            etot2=newetot
        else:
            ReportTest(("Total energy (step %d)" % (i+1,)),
                       newetot, etot2, 20.0)
    print(" *** This test completed ***", file=stdout)

ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
from asap3.md.velocitydistribution import MaxwellBoltzmannDistribution
from asap3 import mpi
from ase.data import reference_states
import numpy as np
import time
import sys

debug = 0
if debug == 1:
    DebugOutput("parallelopenkim%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

openkimmodel = "EMT_Asap_Standard_Jacobsen_Stoltze_Norskov_AlAgAuCuNiPdPt__MO_118428466217_002"

possibleLayouts = [(1,1,2), (1,2,2), (2,2,2), (2,2,3), (2,2,4), (2,3,3),
                   (3,3,3), (3,3,4), (3,4,4), (4,4,4), (4,4,5), (4,5,5),
                   (4,6,6), (6,6,6)]

nbl = "NEIGH_RVEC_H" 
useemt = False

ismaster = mpi.world.rank == 0
isparallel = mpi.world.size > 1

if ismaster:
    print_version(1)

T = 1500  # K   - initial T, real T will be the half.
presteps = 1
steps = 2
logsteps = 1

if isparallel:
    layouts = {}
    for l in possibleLayouts:
        layouts[ l[0]*l[1]*l[2] ] = l
    cpuLayout = layouts[mpi.world.size]
    
def MakeAtoms(elem1, elem2=None):
    if elem2 is None:
        elem2 = elem1
    a1 = reference_states[elem1]['a']
    a2 = reference_states[elem2]['a']
    a0 = (0.5 * a1**3 + 0.5 * a2**3)**(1.0/3.0) * 1.03
    if ismaster:
        # 50*50*50 would be big enough, but some vacancies are nice.
        print("Z1 = %i,  Z2 = %i,  a0 = %.5f" % (elem1, elem2, a0))
        atoms = FaceCenteredCubic(symbol='Cu', size=(51,51,51))
        nremove = len(atoms) - 500000
        assert nremove > 0
        remove = np.random.choice(len(atoms), nremove, replace=False)
        del atoms[remove]
        if isparallel:
            atoms = atoms.repeat(cpuLayout)
        if elem1 != elem2:
            z = atoms.get_atomic_numbers()
            z[np.random.choice(len(atoms), len(atoms)/2, replace=False)] = elem2
            atoms.set_atomic_numbers(z)
    else:
        atoms = None
    if isparallel:
        atoms = MakeParallelAtoms(atoms, cpuLayout)
    MaxwellBoltzmannDistribution(atoms, T * units.kB)
    return atoms
        
def RunTiming(atoms, label):
    dyn = VelocityVerlet(atoms, 5.0*units.fs)
    dyn.attach(MDLogger(dyn, atoms, sys.stderr, peratom=True), interval=logsteps)
    dyn.run(presteps)
    t = time.time()
    dyn.run(steps)
    t = time.time() - t
    t *= 2.0   # Only half a million atoms.
    if ismaster:
        print("%s: %.3f us/(atom*cpu)" % (label, t / steps))
    
def RunAll(name, *args):
    initial = MakeAtoms(*args)
    if isparallel:
        atoms = MakeParallelAtoms(initial, cpuLayout)
    else:
        atoms = Atoms(initial)
    if useemt:
        emt = EMT()
        emt.set_subtractE0(False)
        atoms.set_calculator(emt)
        RunTiming(atoms, name + ' Native EMT')
    else:
        atoms.set_calculator(OpenKIMcalculator(openkimmodel, allowed=nbl,
                                               stress=False, stresses=False))
        RunTiming(atoms, name + " " + nbl)
        
if ismaster:
    print("Running on %d processors" % (mpi.world.size,))    

RunAll("CuAu", 29, 79)
RunAll("Cu", 29)
RunAll("Au", 79)



"""User provided customizations.

This files defines system specific options etc for compiling Asap.

To adapt Asap to a new system type, please add code to this file
to identify it, and contribute the modified file back to the 
Asap project.

Alternatively, make a file called customize-local.py, and put
your customizations there.  

Here are all the lists that can be modified:

NORMAL COMPILATION:
* libraries             Link against these extra libraries
* library_dirs          Search for libraries here (-L)
* include_dirs          Search for include files here (-I)
* extra_link_args       Extra arguments used when linking
* extra_compile_args    Extra arguments used when compilation
* remove_compile_args   Remove these arguments from compilation command line

The following are less likely to be important
* runtime_library_dirs  ???
* extra_objects         Extra object that should be linked (probably useless).
* define_macros         Macros that should be defined
* undef_macros          Macros that should be undefined.

COMPILATION WITH MPI
* mpi_compiler          Tuple of MPI C and C++ compiler driver scripts.

Typically, the following do not need customization.
* mpi_libraries         Extra libraries when compiling with MPI
* mpi_library_dirs      Search for libraries here
* mpi_include_dirs      Search for include files here
* mpi_runtime_library_dirs  ???
* mpi_define_macros     Define these extra macros.

COMPILATION WITH INTEL COMPILER:
* use_intel_compiler    False, True or 'auto'.  Can be overruled on command line.
* intel_compiler        Tuple of Intel C and C++ compiler names
* intel_mpi_compiler    Tuple of command lines that makes the mpi compiler scripts
                        use the Intel compiler.  Should match mpi_compiler.
* intel_compile_args    Command lines flags when compiling.  Overrules extra_compile_args
* intel_libraries       Libraries to link against.  Overrules libraries
* intel_link_args       Command line flags when linking.  Overrules extra_compile_args

COMPILATION WITH INTEL MATH KERNEL LIBRARY (--with-mkl)
* mkl_supported         True if $MKLROOT is defined, or set manually in this file.
* mkl_compile_args      Command line args added to extra_compile_args
* mkl_intel_compile_args  Command line args added to intel_compile_args
* mkl_include_dirs      Extra include directories for MKL
* mkl_libraries         Extra libraries for MKL
* mkl_library_dirs      Extra library_dirs for MKL
* mkl_define_macros     Extra macros to be defined - should at least be ['ASAP_MKL']


To override use the form:
    
    libraries = ['somelib', 'otherlib']

To append use the form

    libraries += ['somelib', 'otherlib']

You can access all variable from the setup.py script.  Useful ones are
* systemname  (for example 'Linux')
* hostname    (may or may not be fully qualified)
* machinename  (The CPU architechture, e.g. x86_64)
* processorname   (May contain more infor than machinename)

"""


if systemname == 'Linux' and machinename == 'x86_64':
    # For performance on 64-bit Linux.
    #
    # WARNING: -march=native is not appropriate to build
    #   binary packages that will run on another machine.
    
    extra_compile_args += ['-ffast-math', 
                           '-march=native', 
                           '-Wno-sign-compare',
                           '-Wno-write-strings']
    remove_compile_args += ['-Wstrict-prototypes']
elif systemname == 'Linux' and machinename == 'i686':
    # For performance on 32-bit Linux.
    
    extra_compile_args += ['-ffast-math', 
                           '-Wno-sign-compare']
    remove_compile_args += ['-Wstrict-prototypes']


# If linking fails with a large number of missing symbols, many of them
# starting with rl_ then try uncommenting the following line.

#libraries += ['readline']



#!/usr/bin/env python
"""
Tests that stresses are calculated correctly by Asap

Name: testStress.py

Description: Part of the Asap test suite.  Tests that stresses are
    calculated correctly by calculating various elastic constants from
    strain and stress and comparing them with the same constants
    calculated using strains and energies.  Can also be imported as a
    module, and used to test advanced calculation methods (QC,
    parallel).

Usage: python testStress.py

Expected result: Some elastic constants for Copper and Silver,
followed by the text 'ALL TESTS SUCCEEDED'.

The elastic constants are calculated by straining the crystal in
various modes, and fitting to the energies or the stresses.
Generally, the energies seems to be most sensitive to numerical noise,
and requires a rather large strain interval (1% or above), whereas the
stresses are much less sensitive to this.  On the other hand,
unlinearities influence the stress fits for large strain intervals.  A
strain interval of [-1%, 1%] is a good compromise, where both methods
work.

C11 and C12 calculated 'directly', i.e. in uniaxial strain, is
sensitive to the strain interval when using the energy to fit C11.
Fitting to the stresses work much better.

C11 and C12 can be calculated in an alternative way using a
volume-conserving deformation and fitting to the energies.

All of the above-mentioned calculations are performed.

"""
from __future__ import print_function

import numpy as np
import sys
from asap3 import *
from asap3.testtools import ReportTest

# When not imported as a module, the following are imported later:
#from Setup.Lattice.FCC111Ortho import *
#from Structures.ChemicalElements import Copper, Silver
#import Structures.ChemicalElements.AtomicWeight
#import Structures.ChemicalElements.CrystalStructure
#from Structures.IonDynamics import VelocityVerlet, Langevin

defaultstrains = 0.01 * np.array((-1, -0.75, -0.5, -0.25, 0.0,
                                         0.25, 0.5, 0.75, 1.0))

def polynomialLeastSquaresFit(parameters, data, max_iterations=None,
                              stopping_limit = 0.0001):
    """Least-square fit to a polynomial.
    
    Least-squares fit to a polynomial whose order is defined by
    the number of parameter values.

    This is a wrapper function replacing the similar function in
    Scientific.Functions.LeastSquares
    """
    order = len(parameters)
    return (np.polyfit(data[:,0], data[:,1], order)[::-1], None)

def makefits(atoms, strains, indices, shear=0):
    """Do the deformations, and get fits to energies and stresses.

    atoms is a list of atoms.
    strains are the strains as floating point numbers
    indices is a list of indices for the strain components to be used.
        For bulk modulus, it will be ((0,0), (1,1), (2,2)) etc.
    """
    energies = []
    stresses = []
    basis = atoms.get_cell()
    vol = np.linalg.det(basis)
    for epsilon in strains:
        if shear:
            adjustment = np.zeros((3,3), np.float)
            for idx in indices:
                adjustment[idx[0]] += idx[2] * epsilon * basis[idx[1]]
            atoms.set_cell(adjustment + basis, scale_atoms=True)
        else:
            scaling = np.ones((3,3), np.float)
            for idx in indices:
                scaling[idx[0]] += idx[1]*epsilon
            atoms.set_cell(scaling * basis, scale_atoms=True)
        energy = atoms.get_potential_energy()
        stress = atoms.get_stress()
        #print ""
        #print epsilon, energy/len(atoms)
        #print stress
        energies.append((epsilon, energy/vol))
        stresses.append((epsilon,) + tuple(stress))
    atoms.set_cell(basis, scale_atoms=True)
    energies = np.array(energies)
    stresses = np.array(stresses)

    #print "Energies:", energies
    energyfit = polynomialLeastSquaresFit((0.0, 0.0, 0.0), energies)
    #print "EnergyFit:", energyfit
    stressfits = []
    for i in range(6):
        stressfits.append(polynomialLeastSquaresFit((0.0, 0.0),
                                                    np.take(stresses,
                                                            (0, i+1), 1)))
    return (energyfit, stressfits)

def findlatticeconst(atoms, latticeconstant):
    """Adjust the volume so the atoms have their lowest energy."""
    basis = atoms.get_cell()
    strains = 0.01 * np.array((-0.2, -0.15, -0.1, -0.05, 0.0, 0.05, 0.1,
                               0.15, 0.2))
    for i in range(5):
        (energyfit, stressfit) = makefits(atoms, strains,
                                          (((0,0), 1), ((1,1), 1), ((2,2),1)))
        pressurefit = -(np.array(stressfit[0][0]) +
                        np.array(stressfit[1][0]) +
                        np.array(stressfit[2][0])) / 3.0
        dilation = -pressurefit[0]/pressurefit[1]
        print("Optimizing lattice constant:", latticeconstant, "->", latticeconstant*(1+dilation/3))
        latticeconstant = latticeconstant*(1+dilation/3)
        basis = (1+dilation/3) * basis
        atoms.set_cell(basis, scale_atoms=True)

def elasticconstants(atoms, name, bookbulk, bookc11, bookc12, bookc44,
                     fitfact=1.0, fitfact2=1.0):
    """Check the elastic constants."""
    (energyfit, stressfits) = makefits(atoms, defaultstrains,
                                       (((0,0), 1), ((1,1), 1), ((2,2),1)))
    bm = 2.0/9.0 * energyfit[0][2] / units.GPa
    bms = []
    for i in range(3):
        bms.append(stressfits[0][0][1] / 3.0 / units.GPa)
    avgbms = np.sum(bms)/3.0
    print("")
    print("Calculation for", name)
    print("  Bulk modulus from energies:", bm)
    print("  Bulk modulus from stresses:", np.sum(bms)/3.0)
    print("  Textbook value:", bookbulk[0])
    print("")
    ReportTest("Bulk modulus (%s, energies)" % name, bm, bookbulk[1],
               0.1*fitfact)
    ReportTest("Bulk modulus (%s, pressure)" % name, avgbms, bm, 0.1*fitfact)
    ReportTest("Bulk modulus (%s, s_xx)" % name, bms[0], avgbms,
               0.00001*avgbms)
    ReportTest("Bulk modulus (%s, s_yy)" % name, bms[1], avgbms,
               0.00001*avgbms)
    ReportTest("Bulk modulus (%s, s_zz)" % name, bms[2], avgbms,
               0.00001*avgbms)
    (energyfit, stressfits) = makefits(atoms, defaultstrains, (((0,0),1),))
    c11 = 2.0 * energyfit[0][2] / units.GPa
    c11s = stressfits[0][0][1] / units.GPa
    c12s = stressfits[1][0][1] / units.GPa
    print("")
    print("Calculation for", name)
    print("  C_11 from energies:", c11)
    print("  C_11 from stress:", c11s)
    print("  C_12 from stress:", c12s)
    print("    Bulk modulus from C_11 and C_12:", (c11s + 2 * c12s) / 3.0)
    print("    Textbook values: C_11 = %.1f; C_12 = %.1f" % (bookc11[0],
                                                             bookc12[0]))
    (energyfit, stressfits) = makefits(atoms, defaultstrains,
                                       (((0,0),1), ((1,1), -0.5),
                                        ((2,2), -0.5)))
    # C11 - C12
    c11mc12 = 4.0/3.0 * energyfit[0][2] / units.GPa
    # B =  (C11 + 2 C12)/3
    altc11 = (3*bm + 2*c11mc12) / 3.0
    altc12 = (3*bm - c11mc12) / 3.0
    print("  C_11 from alternative energies:", altc11)
    print("  C_12 from alternative energies:", altc12)
    print("    Bulk modulus from C_11 and C_12:", (altc11 + 2 * altc12) / 3.0)
    print("")
    
    ReportTest("C11 from stress", c11s, bookc11[1], 0.1*fitfact)
    ReportTest("C11 from energies", c11, c11s, 0.1*fitfact)
    ReportTest("C12 from stress", c12s, bookc12[1], 0.1*fitfact)
    ReportTest("B from C11 and C12", (c11s + 2 * c12s) / 3.0, bookbulk[1],
               0.5*fitfact)
    ReportTest("C11 from alt. energies", altc11, c11s, 0.5*fitfact)
    ReportTest("C12 from alt. energies", altc12, c12s, 0.5*fitfact)
    print("")

    (energyfit, stressfits) = makefits(atoms, defaultstrains,
                                       (((2,1), (2,2), 1),
                                        ((1,2), (1,1), 1)), shear=1)
    c44 = 0.5 * energyfit[0][2] / units.GPa
    c44s = 0.5 * stressfits[3][0][1] / units.GPa
    print("")
    print("Calculation for", name)
    print("  C_44 from energies:", c44)
    print("  C_44 from stresses:", c44s)
    print("  Textbook value:", bookc44[0])
    print("    (please do not expect good agreement for the usual EMT potential)")
    print("")
    ReportTest("C44 from energies", c44, bookc44[1], 0.1*fitfact)
    ReportTest("C44 from stresses", c44s, c44, 0.2*fitfact)
    nulls = (("C14", 0), ("C24", 1), ("C34", 2), ("C45", 4), ("C46", 5))
    print("Testing that the non-existing elastic constants are indeed zero:")
    for name, idx in nulls:
        cnull = stressfits[idx][0][1] / units.GPa
        #print cnull
        ReportTest(name+" from stresses", cnull, 0.0, 0.00001)
    print("")
    print("Testing other shear modes:")
    (energyfit, stressfits) = makefits(atoms, defaultstrains,
                                       (((2,0), (2,2), 1),
                                        ((0,2), (0,0), 1)), shear=1)
    c44x = 0.5 * energyfit[0][2] / units.GPa
    c44sx = 0.5 * stressfits[4][0][1] / units.GPa
    ReportTest("C44(alt) from energies", c44x, c44, 0.2*fitfact2)
    ReportTest("C44(alt) from stresses", c44sx, c44s, 0.2*fitfact2)
    nulls = (("C14", 0), ("C24", 1), ("C34", 2), ("C45", 3), ("C46", 5))
    for name, idx in nulls:
        cnull = stressfits[idx][0][1] / units.GPa
        ReportTest(name+"(alt) from stresses", cnull, 0.0, 0.00001)
    (energyfit, stressfits) = makefits(atoms, defaultstrains,
                                       (((1,0), (1,1), 1),
                                        ((0,1), (0,0), 1)), shear=1)
    c44x = 0.5 * energyfit[0][2] / units.GPa
    c44sx = 0.5 * stressfits[5][0][1] / units.GPa
    ReportTest("C44(alt) from energies", c44x, c44, 0.1*fitfact)
    ReportTest("C44(alt) from stresses", c44sx, c44s, 0.1*fitfact)
    nulls = (("C14", 0), ("C24", 1), ("C34", 2), ("C45", 3), ("C46", 4))
    for name, idx in nulls:
        cnull = stressfits[idx][0][1] / units.GPa
        ReportTest(name+"(alt) from stresses", cnull, 0.0, 0.00001)
    print("")
                                       
book = {}
book['Cu'] = {"bookbulk": (134.3, 133.52),
              "bookc11": (168.3, 171.65),
              "bookc12": (122.1, 114.82),
              "bookc44": (75.7, 88.45)}

book['Copper_Rasmussen'] = {"bookbulk": (134.3, 140.17),
                            "bookc11": (168.3, 172.75),
                            "bookc12": (122.1, 124.32),
                            "bookc44": (75.7, 80.39)}

book['Ag'] = {"bookbulk": (103.8, 99.01),
              "bookc11": (124.0, 124.45),
              "bookc12": (93.7, 86.27),
              "bookc44": (46.12, 53.25)}

if __name__ == "__main__":
    print("This it not a test, but a module imported from a few tests.")
    



# VelocityDistributions.py -- set up a velocity distribution

"""Module for setting up e.g. Maxwell-Boltzmann velocity distributions.

Currently, only one function is defined, MaxwellBoltzmannDistribution,
which sets the momenta of a list of atoms according to a
Maxwell-Boltzmann distribution at a given temperature.
"""

import Numeric
import RandomArray

def _maxwellboltzmanndistribution(masses, temp):
    xi = RandomArray.standard_normal(shape=(len(masses),3))
    momenta = xi * Numeric.sqrt(masses * temp)[:,Numeric.NewAxis]
    return momenta

def MaxwellBoltzmannDistribution(atoms, temp):
    """Sets the momenta to a Maxwell-Boltzmann distribution."""
    momenta = _maxwellboltzmanndistribution(atoms.GetMasses(), temp)
    atoms.SetCartesianMomenta(momenta)

    



from __future__ import print_function
#import production

from asap3 import *
from asap3.md.verlet import VelocityVerlet
from numpy import *
from pickle import *
#from Asap.Trajectories.NetCDFTrajectory import *
import sys, os, time
from asap3.testtools import *
from asap3.mpi import world

debug = 0
if debug == 1:
    DebugOutput("makeverlet%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

print_version(1)
set_verbose(0)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

def report():
    for i in range(world.size):
        if i == world.rank:
            print("Data on processor", i)
            for key in atoms.arrays.keys():
                print("  ", key, atoms.arrays[key].shape)
            r = atoms.get_positions()
            if len(r):
                print("Limits to the positions:")
                print ("[%.4f, %.4f]  [%.4f, %.4f]  [%.4f, %.4f]" %
                       (min(r[:,0]), max(r[:,0]), min(r[:,1]), max(r[:,1]),
                        min(r[:,2]), max(r[:,2])))
            if world.size > 1:
                print("Ghost data on processor", i)
                for key in atoms.ghosts.keys():
                    print("  ", key, atoms.ghosts[key].shape)
                r = atoms.ghosts['positions']
                if len(r):
                    print("Limits to the ghost positions:")
                    print ("[%.4f, %.4f]  [%.4f, %.4f]  [%.4f, %.4f]" %
                           (min(r[:,0]), max(r[:,0]), min(r[:,1]), max(r[:,1]),
                            min(r[:,2]), max(r[:,2])))
        world.barrier()


# Ensure same time step as in Asap-2
timeunit = 1.018047e-14             # Seconds
femtosecond = 1e-15 / timeunit      # Marginally different from units.fs

if isparallel:
    print("RUNNING PARALLEL VERSION OF TEST SCRIPT")
else:
    print("RUNNING SERIAL VERSION OF TEST SCRIPT")

if ismaster:
    # First, load a small system
    if os.path.exists("Verlet_legacymass.pickle"):
        picfile = "Verlet_legacymass.pickle"
    else:
        picfile = "../Verlet_legacymass.pickle"
    try:
        data = load(open(picfile, "rb"))
    except UnicodeDecodeError:
        data = load(open(picfile, "rb"), encoding='latin1')
    init_pos = array(data["initial"])
    init_pos.shape = (-1,3)
    init_box = array(data["box"])
    init_box.shape = (3,3)
    a = Atoms(positions=init_pos, cell=init_box, pbc=(1,1,1))
    a.set_atomic_numbers(29*ones(len(a)))
    #atoms = a.repeat((2,2,3))
    # Ensure same order of atoms as in ASE-2
    atoms = a.repeat((1,1,3)).repeat((1,2,1)).repeat((2,1,1))
    dx = 0.1 * sin(arange(3*len(atoms))/10.0)
    dx.shape = (-1,3)
    atoms.set_positions(atoms.get_positions() + dx)
    del dx
    out = sys.stderr
else:
    atoms = None
    out = open("/dev/null", "w")

if isparallel:
    atoms = MakeParallelAtoms(atoms, cpulayout)
    nTotalAtoms = atoms.get_number_of_atoms()
else:
    nTotalAtoms = len(atoms)

#report()

print("Setting potential")
atoms.set_calculator(EMT())

dyn = VelocityVerlet(atoms, 2 * femtosecond)
# if isparallel:
#     traj = ParallelNetCDFTrajectory("ptraj.nc", atoms, interval=20)
# else:
#     traj = NetCDFTrajectory("ptraj.nc", atoms, interval=20)
# dyn.Attach(traj)
# traj.Update()

print("Number of atoms:", nTotalAtoms)

epot = atoms.get_potential_energy() / nTotalAtoms
ekin = atoms.get_kinetic_energy() / nTotalAtoms
etotallist = [epot+ekin]
ekinlist = [ekin]

#report()

if ismaster:
    print("\nE_pot = %-12.5f  E_kin = %-12.5f  E_tot = %-12.5f" % (epot, ekin,
                                                                 epot+ekin))
ReportTest("Initial potential energy", epot, 0.04312, 1e-4)
ReportTest("Initial kinetic energy", ekin, 0.0, 1e-9)

dyn.attach(MDLogger(dyn, atoms, "-", peratom=True), interval=10)

for i in range(40):
    dyn.run(5)
    epot = atoms.get_potential_energy() / nTotalAtoms
    ekin = atoms.get_kinetic_energy() / nTotalAtoms
    etotallist.append(epot+ekin)
    ekinlist.append(ekin)

if ismaster:
    print("Average total energy:", sum(etotallist)/len(etotallist))
    print("Average kinetic energy:", sum(ekinlist)/len(ekinlist))

ReportTest("Agv. total energy", sum(etotallist)/len(etotallist), 0.04309,
           0.0001)
ReportTest("Agv. kinetic energy", sum(ekinlist)/len(ekinlist), 0.02165,
           0.002)
ReportTest.Summary(0)




from __future__ import print_function
from asap3 import *
from numpy import *
from asap3.testtools import ReportTest
from ase.lattice.cubic import *
from OpenKIM_modelname import openkimmodel

element = 'Au'

energy = None
force = None

if OpenKIMsupported:
    sizes = arange(10,0,-1)
    for s in sizes:
        atoms = FaceCenteredCubic(symbol=element, size=(s,s,s), pbc=True)
        r = atoms.get_positions()
        for i in range(0, len(r), 4):
            r[i][0] += 0.1
        atoms.set_positions(r)
        try:
            atoms.set_calculator(OpenKIMcalculator(openkimmodel))
        except AsapError as oops:
            if oops.args[0].startswith('Failed to initialize OpenKIM model'):
                print("OpenKIM model {} not installed - skipping test.".format(openkimmodel))
                break
            else:
                raise
        e = atoms.get_potential_energy() / len(atoms)   
        print("%5i atoms: E = %.5f eV/atom" % (len(atoms), e))
        if energy is None:
            energy = e
        else:
            ReportTest("Energy for size %i (%i atoms)" % (s, len(atoms)),
                       e, energy, 1e-8)
        f = atoms.get_forces()
        if force is None:
            force = f[:4]
        else:
            for i in range(len(f)):
                for j in range(3):
                    ReportTest("Force for size %i atom %i of %i component %i"
                               % (s, i, len(atoms), j),
                               f[i,j], force[i % 4, j], 1e-8, silent=True)
            #print force
            #print f[:10]
        del atoms

ReportTest.Summary()



from __future__ import print_function
import numpy as np

from ase.optimize import LBFGS, LBFGSLineSearch
from ase.lattice.surface import fcc100, fcc110, fcc111, hcp0001

def SurfaceEnergy(images, natoms, calc, fmax=0.01, debug=False):
    """Calculate the surface energy from a list of slab images.

    Returns the surface energy in eV/atom.
    
    Parameters
    ----------
    images: List of slab images which the calculation is based on. The x and
    y dimensions of the images unit cell must be conserved.

    natoms: Number of atoms in a atomic layer in the slabs.

    calc: Calculator object that can be attached to the images.
    """
    ucell = images[0].get_cell()

    layers = np.zeros(len(images))
    energies = np.zeros(len(images))

    if debug:
        print("Layers   Energy   LBFGS steps")

    for i, atoms in enumerate(images):
        cell = atoms.get_cell()
        if (ucell[0] != cell[0]).any() or (ucell[1] != cell[1]).any():
            raise ValueError("The x and y dimensions of the unit cell must be conserved.")

        atoms.set_calculator(calc)

        dyn = LBFGS(atoms, logfile=None, trajectory=None)
        dyn.run(fmax=fmax, steps=1000)
        assert dyn.converged(), "LBFGS not converged in 100 steps!"

        layers[i] = len(atoms) / natoms
        energies[i] = atoms.get_potential_energy()

        if debug:
            print("%4i%12.3f%10i" % (layers[i], energies[i], dyn.get_number_of_steps()))

    p = np.polyfit(layers, energies, 1)
    surface_energy = p[-1] / (2 * natoms)
    assert surface_energy > 0.0

    return surface_energy

def get_surface_slabs(symbol, surfacetype, surfacesize, layers, latticeconstants,
                      vacuum=10.0):
    images = []
    if surfacetype == 'fcc100' or surfacetype == 'fcc111':
        nlayeratoms = surfacesize[0] * surfacesize[1]
        if isinstance(latticeconstants, (list, tuple)):
            if not len(latticeconstants) == 1:
                raise ValueError("Too many latticeconstants")
            a = latticeconstants[0]
        else:
            a = latticeconstants
    elif surfacetype == 'hcp0001':
        nlayeratoms = surfacesize[0] * surfacesize[1]
        if isinstance(latticeconstants, (list, tuple)):
            if not len(latticeconstants) == 2:
                raise ValueError("Latticeconstants must be on the form [a, c]")
            a = latticeconstants[0]
            c = latticeconstants[1]
        else:
            raise ValueError("Latticeconstants must be on the form [a, c]")
    else:
        raise ValueError("Surface type '%s' is not supported" % (surfacetype,))


    for n in layers:
        size = surfacesize + (n,)
        if surfacetype == 'fcc100':
            images.append(fcc100(symbol=symbol, size=size, a=a, vacuum=vacuum))
        elif surfacetype == 'fcc111':
            images.append(fcc111(symbol=symbol, size=size, a=a, vacuum=vacuum,
                                 orthogonal=True))
        elif surfacetype == 'hcp0001':
            images.append(hcp0001(symbol=symbol, size=size, a=a, c=c,
                                  vacuum=vacuum, orthogonal=True))

    return images, nlayeratoms

if __name__ == '__main__':
    from asap3 import EMT
    from ase.data import reference_states

    debug = False
    layers = np.arange(5, 14, 2)

    print("Calculating (100) surface energies with EMT")
    images, nlayeratoms = get_surface_slabs('Cu', 'fcc100', (6, 6), layers,
                                            reference_states[29]['a'])
    surface_energy = SurfaceEnergy(images, nlayeratoms, EMT(), debug=debug)
    print("Cu:", surface_energy)
    images, nlayeratoms = get_surface_slabs('Ag', 'fcc100', (6, 6), layers,
                                            reference_states[47]['a'])
    surface_energy = SurfaceEnergy(images, nlayeratoms, EMT(), debug=debug)
    print("Ag:", surface_energy)
    images, nlayeratoms = get_surface_slabs('Au', 'fcc100', (6, 6), layers,
                                            reference_states[79]['a'])
    surface_energy = SurfaceEnergy(images, nlayeratoms, EMT(), debug=debug)
    print("Au:", surface_energy)

    print("\nCalculating (111) surface energies with EMT")
    images, nlayeratoms = get_surface_slabs('Cu', 'fcc111', (6, 6), layers,
                                            reference_states[29]['a'])
    surface_energy = SurfaceEnergy(images, nlayeratoms, EMT(), debug=debug)
    print("Cu:", surface_energy)
    images, nlayeratoms = get_surface_slabs('Ag', 'fcc111', (6, 6), layers,
                                            reference_states[47]['a'])
    surface_energy = SurfaceEnergy(images, nlayeratoms, EMT(), debug=debug)
    print("Ag:", surface_energy)
    images, nlayeratoms = get_surface_slabs('Au', 'fcc111', (6, 6), layers,
                                            reference_states[79]['a'])
    surface_energy = SurfaceEnergy(images, nlayeratoms, EMT(), debug=debug)
    print("Au:", surface_energy)





from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
from asap3.EMT2013Parameters import sihb_PtY_parameters

def MakeLJ():
    "Lennard-Jones potential."
    p = LennardJones([78], [0.15], [2.35], -1.0, False)
    return p

def MakeEMT2013():
    "EMT2013 potential."
    return EMT2013(sihb_PtY_parameters)

for pot in (EMT, MakeLJ, MakeEMT2013):
    print("Testing:", pot.__doc__.split("\n")[0])
    
    atoms1 = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                              size=(6,6,6), symbol="Pt")
    emt = pot()
    atoms1.set_calculator(emt)
    f1 = atoms1.get_forces()


    atoms2 = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                              size=(5,5,5), symbol="Pt")
    atoms2.set_calculator(emt)
    f3 = atoms1.get_forces()
    maxdev = abs(f3 - f1).max()
    print(maxdev)
    ReportTest("Max error 1:", maxdev, 0.0, 1e-6)

    atoms1 = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                              size=(6,6,6), symbol="Pt")
    emt = pot()
    atoms1.set_calculator(emt)
    f1 = atoms1.get_forces()


    atoms2 = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                              size=(5,5,5), symbol="Pt")
    atoms2.set_calculator(emt)
    atoms3 = atoms2.copy()
    atoms3.set_calculator(pot())
    f0 = atoms3.get_forces()
    f2 = atoms2.get_forces()
    f3 = atoms1.get_forces()
    maxdev = abs(f2 - f0).max()
    print(maxdev)
    ReportTest("Max error 2:", maxdev, 0.0, 1e-6)
    maxdev = abs(f3 - f1).max()
    print(maxdev)
    ReportTest("Max error 3:", maxdev, 0.0, 1e-6)

ReportTest.Summary()



"Test the RawRadialDistributionFunction function."
from __future__ import print_function

from numpy import *
from asap3 import *
from asap3 import _asap
from ase.lattice.compounds import NaCl
from asap3.testtools import *
from asap3.Internal.ListOfElements import ListOfElements

print_version(1)

# testtypes: latticeconst, maxRDF, bins, useEMT
testtypes = ((5.1, 6.001, 100, False),
             (5.2, 6.001, 100, True),
             (5.1, 15.001, 100, False),
             (5.15, 15.001, 100, True))

for latconst, maxrdf, nbins, withemt in testtypes:

    atoms = NaCl(directions=[[1,0,0],[0,1,0],[0,0,1]], symbol=("Cu","Au"),
                 size=(10,10,10), latticeconstant=latconst, debug=0)
    natoms = len(atoms)
    ReportTest("Number of atoms", natoms, 8000, 0)

    if withemt:
        atoms.set_calculator(EMT())
        print(atoms.get_potential_energy())

    result = _asap.RawRDF(atoms, maxrdf, nbins, zeros(len(atoms), int32), 1,
                          ListOfElements(atoms))
    z = atoms.get_atomic_numbers()[0]

    globalrdf, rdfdict, countdict = result

    print(globalrdf)
    localrdf = zeros(globalrdf.shape)
    for i in (29,79):
        for j in (29,79):
            x = rdfdict[0][(i,j)]
            print("LOCAL", i, j)
            print(x)
            localrdf += x


    ReportTest("Local and global RDF are identical",
               min( globalrdf == localrdf), 1, 0)
    ReportTest("Atoms are counted correctly",
               countdict[0][29] + countdict[0][79], natoms, 0)
    
    shellpop = [6, 12, 8, 6, 24, -1]
    shell = [sqrt(i+1.0)/2.0 for i in range(6)]
    print(shell)
    print(shellpop)
    n = 0

    dr = maxrdf/nbins

    for i in range(nbins):
        if (i+1)*dr >= shell[n] * latconst:
            if shellpop[n] == -1:
                print("Reached the end of the test data")
                break
            ReportTest(("Shell %d (%d)" % (n+1, i)), globalrdf[i],
                       natoms*shellpop[n], 0)
            n += 1
        else:
            ReportTest(("Between shells (%d)" % (i,)), globalrdf[i], 0, 0)

    
ReportTest.Summary()




"""Molecular Dynamics."""

import numpy as np

try:
    from ase.optimize.optimize import Dynamics
except ImportError:
    # Fallback to old placement
    from ase.optimize import Dynamics
from ase.data import atomic_masses
from ase.md import MDLogger

_counter = 0  # Prevents identical prefixes


class ParallelMolDynMixin:
    def __init__(self, prefix, atoms):
        global _counter
        self.prefix = prefix+str(_counter)+"_"
        _counter += 1
        self._uselocaldata = not getattr(atoms, "parallel", False)
        self._localdata = {}
        
    def set(self, name, var):
        """Set a local variable.

        If the local variable is a scalar, it is stored locally.  If
        it is an array it is stored on the atoms.  This allows for
        parallel Asap simulations, where such arrays will have to
        migrate among processors along with all other data for the
        atoms.
        """
        if self._uselocaldata or getattr(var, "shape", ()) == ():
            self._localdata[name] = var
        else:
            if name in self._localdata:
                del self._localdata[name]
            self.atoms.set_array(self.prefix+name, var)

    def get(self, name):
        try:
            return self._localdata[name]
        except KeyError:
            return self.atoms.get_array(self.prefix+name, copy=False)



from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from ase.lattice.cubic import FaceCenteredCubic, BodyCenteredCubic
from . import quat_utils
import numpy as np
import itertools

# DELAYED IMPORTS (may not always be available):
#
# scipy.spatial
# matplotlib
# mpl_toolkits



def random_rotations(ngrains):
    """Return random grain rotations for all grains.

    Parameters:
    grains:  The number of grains.

    Returns:
    A list of 3*3 rotation matrices taken from a uniform distribution of
    rotations.
    """
    return [quat_utils.random_rotation_matrix() for i in range(ngrains)]


def bcc_grains(gridsize, perturbation=0.0):
    """Creates a grain layout based on a perturbed BCC lattice.

    Creates a set of grain centers in scaled space (i.e. with coordinates
    between 0.0 and 1.0), based on a regular BCC lattice, possibly with a
    random perturbation added.

    Parameters:
    gridsize: Size of the BCC lattice in lattice units.  Either an integer,
    or a tuple of three integers.

    perturbation: If non-zero, all grain centers are perturbed by a random
    amount drawn from a normal distribution with this spread.

    Returns:
    A N*3 numpy array of grain centers.
    """
    graincenters = BodyCenteredCubic(symbol='H', latticeconstant=1.0, size=grainsize)
    graincenters = graincenters.get_positions() + 0.25
    graincenters /= grainsize
    pert = np.random.standard_normal(graincenters.shape) * perturbation
    graincenters += pert
    return graincenters

###
### OPTIMIZING VORONOI TESSELATIONS.
###

def lloyd_step(centers, box, resolution=20, alpha=1.0, verbose=True):
    """A single step of Lloyd's algorithm towards a centroid Voronoi tesselation.

    See https://en.wikipedia.org/wiki/Lloyds_algorithm
    
    Parameters:
    centers: List of Voronoi centers (generators).  N*3 numpy array.
    
    box: Three numbers, giving the size of the computational box (PBC!)
    
    resolution: Use 2**resolution points in the Monte Carlo
        evaluation of the centroids.
        
    alpha: Acceleration factor in Lloyd's algorithm.  Values slightly below 2
        are supposedly good.  The default (1.0) gives slow convergence.
    
    Returns:
    Updated Voronoi centers (same shape as centers parameter).
    """
    npoints = len(centers)
    assert centers.shape == (npoints, 3)
    box = np.array(box)
    assert box.shape == (3,)
    
    # Generate the periodic images
    generators = periodic_images(centers, box)
    assert generators.shape == (27*npoints, 3)
    
    # Generate random point distribution
    randsize = (1 << resolution)
    if verbose:
        print("Size of random pool: {}  ({:.1e})".format(randsize, randsize))
    rpoints = np.transpose([np.random.uniform(-box[i], 2*box[i], randsize) for i in range(3)])
    assert rpoints.shape == (randsize, 3), rpoints.shape
    #print(rpoints)
    
    if verbose:
        print("Assigning to Voronoi cells.")
    belongs_to = [np.argmin( ((p - generators)**2).sum(axis=1)) for p in rpoints]
    belongs_to = np.array(belongs_to)
    
    # Calculate the new centroids (only for the original Voronoi centers)
    if verbose:
        print("Calculating centroids.")
    centroids = np.zeros((npoints, 3))
    for i in range(npoints):
        mypoints = (belongs_to == i)
        centroids[i] = rpoints[mypoints].sum(axis=0) / mypoints.sum()

    # Acceleration of convergence
    centroids = alpha * centroids - (alpha-1) * centers

    # Wrap by PBC - not attempting to do it elegantly
    for i in range(len(centroids)):
        for j in range(3):
            if centroids[i,j] < 0.0:
                centroids[i,j] += box[j]
            if centroids[i,j] >= box[j]:
                centroids[i,j] -= box[j]

    return centroids

###
### EVALUATING AND PLOTTING VORONOI TESSELATIONS
###

def evaluate_voronoi_cells(centers, box):
    """Calculate four parameters evaluating a Voronoi tesselation.

    These four quantities are calculated:
    
    The cell surface area is a measure of some kind of quality/regularity
    of the tesselation, and should decrease is Lloyd's algorithm is applied.

    The total cell volume should be the volume of the box, and is only
    included as a sanity check.

    The largest lateral scaled distance from a cell center to an edge
    measures how close the most critial Voronoi cell is from touching
    itself through the periodic boundary conditions.  The value is 0.5
    if a cell touches itself, values below shows how far it is from doing
    so.

    The number of Voronoi cells touching themselves through the periodic
    boundary conditions.

    
    Parameters:
    centers: A N*3 numpy array of the grain centers (in real space).

    box: The size of the computational box (a 3-tuple).  It is assumed that
        the centers are inside the box, or at most slightly outside.

    Returns:
    A 4-tuple: (total interfacial area; total cell volume; max lateral
    scaled distance to cell boundary; number of self-touching cells)
    """
    import scipy.spatial
    
    npoints = len(centers)
    generators = periodic_images(centers, box)
    voronoi = scipy.spatial.Voronoi(generators)
    regions = [voronoi.regions[voronoi.point_region[i]] for i in range(npoints)]
    for r in regions:
        assert -1 not in r
    # Calculate surface volume and area
    volume = area = 0.0
    for reg in regions:
        cell_vertices = np.array([voronoi.vertices[v] for v in reg])
        hull = scipy.spatial.ConvexHull(cell_vertices)
        volume += hull.volume
        area += hull.area
    # Check if any cell sees itself by calculating the
    # max distance to the vertices in the L1 norm
    maxdist = 0.0
    biggrains = 0
    for i in range(npoints):
        reg = regions[i]
        ctr = centers[i]
        cell_vertices = np.array([voronoi.vertices[v] for v in reg])
        maxdistgrain = 0
        for vert in cell_vertices:
            d = np.abs((vert - ctr)/box).max()
            if d > maxdistgrain:
                maxdistgrain = d
        if maxdistgrain > maxdist:
            maxdist = maxdistgrain
        if maxdistgrain > 0.4999:
            biggrains += 1
    return area, volume, maxdist, biggrains

def plot_voronoi_cells(centers, box, show=True, cellperplot=5):
    """Plot Voronoi tesselations.

    Plots a Voronoi tesselation (nanocrystalline grain structure)
    using matplotlib.

    Parameters:
    centers: A N*3 numpy array of the grain centers (in real space).

    box: The size of the computational box (a 3-tuple).  It is assumed that
        the centers are inside the box, or at most slightly outside.

    show=True: Set to False to skip the call to matplotlib.pyplot.show().
    
    Returns:
    A list-of-figures and a list-of-axes.  They are lists of matplotlib
    figure and Axes3D objects, respectively.
    """
    
    import scipy.spatial
    from mpl_toolkits.mplot3d import Axes3D
    from mpl_toolkits.mplot3d.art3d import Poly3DCollection
    import matplotlib.pyplot as plt
    from matplotlib.colors import colorConverter

    npoints = len(centers)
    generators = periodic_images(centers, box)
    voronoi = scipy.spatial.Voronoi(generators)
    regions = [voronoi.regions[voronoi.point_region[i]] for i in range(len(centers))]
    for r in regions:
        assert -1 not in r
    colors = 'rbgyc'
    axlist = []
    figlist = []
    for i, reg in enumerate(regions[:npoints]):
        if i % cellperplot == 0:
            fig = plt.figure()
            ax = Axes3D(fig)
            figlist.append(fig)
            axlist.append(ax)
        cell_vertices = np.array([voronoi.vertices[v] for v in reg])
        hull = scipy.spatial.ConvexHull(cell_vertices)
        verts = [hull.points[simplex] for simplex in hull.simplices]
        poly = Poly3DCollection(verts, facecolors = colorConverter.to_rgba(
            colors[i % cellperplot], alpha=0.6))
        ax.add_collection3d(poly)
        ax.set_xlim3d(0,100.)
        ax.set_ylim3d(0,100.)
        ax.set_zlim3d(0,100.)
    plt.show()
    return figlist, axlist




###
###  Helper tools
###

def periodic_images(points, box):
    points = np.array(points)
    box = np.array(box)
    result = [points]
    for delta in itertools.product((-1,0,1), repeat=3):
        if delta == (0, 0, 0):
            continue # Do not repeat original points.
        newpoints = points + (delta * box)
        result.append(newpoints)
    return np.concatenate(result)



"""Langevin dynamics class."""

import numpy as np
from numpy.random import standard_normal, randint
import asap3
import ase
from asap3.md.md import ParallelMolDynMixin
from ase.md.md import MolecularDynamics
from ase.md.langevin import Langevin as Langevin_ASE
import asap3.constraints
import sys

class ASE_Langevin_v2(Langevin_ASE, ParallelMolDynMixin, object):
    def __init__(self, atoms, timestep, temperature, friction, fixcm=True,
                 trajectory=None, logfile=None, loginterval=1, seed=None):
        assert(getattr(Langevin_ASE, "_lgv_version", 1) == 2)
        ParallelMolDynMixin.__init__(self, "Langevin", atoms)
        Langevin_ASE.__init__(self, atoms, timestep, temperature, friction, fixcm,
                              trajectory, logfile, loginterval, communicator=None)

    sdpos = property(lambda s: s.get("sdpos"), lambda s, x: s.set("sdpos", x))
    sdmom = property(lambda s: s.get("sdmom"), lambda s, x: s.set("sdmom", x))
    c1 = property(lambda s: s.get("c1"), lambda s, x: s.set("c1", x))
    c2 = property(lambda s: s.get("c2"), lambda s, x: s.set("c2", x))
    act0 = property(lambda s: s.get("act0"), lambda s, x: s.set("act0", x))
    c3 = property(lambda s: s.get("c3"), lambda s, x: s.set("c3", x))
    c4 = property(lambda s: s.get("c4"), lambda s, x: s.set("c4", x))
    pmcor = property(lambda s: s.get("pmcor"), lambda s, x: s.set("pmcor", x))
    cnst = property(lambda s: s.get("cnst"), lambda s, x: s.set("cnst", x))
    masses = property(lambda s: s.get("masses"), lambda s, x: s.set("masses", x))

class ASE_Langevin_v3(Langevin_ASE, ParallelMolDynMixin, object):
    def __init__(self, atoms, timestep, temperature, friction, fixcm=True,
                 trajectory=None, logfile=None, loginterval=1, seed=None):
        assert(getattr(Langevin_ASE, "_lgv_version", 1) == 3)
        ParallelMolDynMixin.__init__(self, "Langevin", atoms)
        Langevin_ASE.__init__(self, atoms, timestep, temperature, friction, fixcm,
                              trajectory, logfile, loginterval, communicator=None)

    def _get_com_velocity(self):
        """Return the center of mass velocity."""
        if getattr(self.atoms, "parallel", False):
            data = np.zeros(4)
            data[:3] = np.dot(self.masses.flatten(), self.v)
            data[3] = self.masses.sum()
            self.atoms.comm.sum(data)
            return data[:3] / data[3]
        else:
            return np.dot(self.masses.flatten(), self.v) / self.masses.sum()

    temp = property(lambda s: s.get("temp"), lambda s, x: s.set("temp", x))
    fr = property(lambda s: s.get("fr"), lambda s, x: s.set("fr", x))
    masses = property(lambda s: s.get("masses"), lambda s, x: s.set("masses", x))
    c1 = property(lambda s: s.get("c1"), lambda s, x: s.set("c1", x))
    c2 = property(lambda s: s.get("c2"), lambda s, x: s.set("c2", x))
    c3 = property(lambda s: s.get("c3"), lambda s, x: s.set("c3", x))
    c4 = property(lambda s: s.get("c4"), lambda s, x: s.set("c4", x))
    c5 = property(lambda s: s.get("c5"), lambda s, x: s.set("c5", x))
    v = property(lambda s: s.get("v"), lambda s, x: s.set("v", x))
    xi = property(lambda s: s.get("xi"), lambda s, x: s.set("xi", x))
    eta = property(lambda s: s.get("eta"), lambda s, x: s.set("eta", x))

class Langevin_Fast(MolecularDynamics, ParallelMolDynMixin, object):
    def __init__(self, atoms, timestep, temperature, friction, fixcm=True,
                 trajectory=None, logfile=None, loginterval=1, seed=None):
        assert(getattr(Langevin_ASE, "_lgv_version", 1) >= 2)
        ParallelMolDynMixin.__init__(self, "Langevin", atoms)
        self._uselocaldata = False # Need to store on atoms for serial simul too.
        self.calculator = atoms.get_calculator()
        if not atoms.has('momenta'):
            atoms.set_momenta(np.zeros((len(atoms), 3), float))
        if atoms.constraints:
            assert len(atoms.constraints) == 1
            constraint = atoms.constraints[0]
            assert isinstance(constraint, asap3.constraints.FixAtoms)
            constraint.prepare_for_asap(atoms)
            # Make all constants arrays by making friction an array
            friction = friction * np.ones(len(atoms))
            fixcm = False   # Unneccesary (and incompatible) when FixAtoms constraint used.
        if seed is None:
            seed = randint(1 << 30)
        assert isinstance(seed, int), "seed must be an int"
        self.asap_md = asap3._asap.Langevin(atoms, self.calculator, timestep,
                                            self.prefix+"sdpos", self.prefix+"sdmom",
                                            self.prefix+"c1", self.prefix+"c2",
                                            fixcm, seed)
        MolecularDynamics.__init__(self, atoms, timestep, trajectory,
                                   logfile, loginterval)
        self.temp = temperature
        self.frict = friction
        self.fixcm = fixcm  # will the center of mass be held fixed?
        self.communicator = None
        self.updatevars()

    def set_temperature(self, temperature):
        self.temp = temperature
        self.updatevars()

    def set_friction(self, friction):
        self.frict = friction
        self.updatevars()

    def set_timestep(self, timestep):
        self.dt = timestep
        self.updatevars()

    def updatevars(self):
        dt = self.dt
        # If the friction is an array some other constants must be arrays too.
        self._localfrict = hasattr(self.frict, 'shape')
        lt = self.frict * dt
        masses = self.masses
        sdpos = dt * np.sqrt(self.temp / masses.reshape(-1) *
                             (2.0 / 3.0 - 0.5 * lt) * lt)
        sdpos.shape = (-1, 1)
        sdmom = np.sqrt(self.temp * masses.reshape(-1) * 2.0 * (1.0 - lt) * lt)
        sdmom.shape = (-1, 1)
        pmcor = np.sqrt(3.0) / 2.0 * (1.0 - 0.125 * lt)
        cnst = np.sqrt((1.0 - pmcor) * (1.0 + pmcor))

        act0 = 1.0 - lt + 0.5 * lt * lt
        act1 = (1.0 - 0.5 * lt + (1.0 / 6.0) * lt * lt)
        act2 = 0.5 - (1.0 / 6.0) * lt + (1.0 / 24.0) * lt * lt
        c1 = act1 * dt / masses.reshape(-1)
        c1.shape = (-1, 1)
        c2 = act2 * dt * dt / masses.reshape(-1)
        c2.shape = (-1, 1)
        c3 = (act1 - act2) * dt
        c4 = act2 * dt
        del act1, act2
        if self._localfrict:
            # If the friction is an array, so are these
            act0.shape = (-1, 1)
            c3.shape = (-1, 1)
            c4.shape = (-1, 1)
            pmcor.shape = (-1, 1)
            cnst.shape = (-1, 1)
        self.sdpos = sdpos
        self.sdmom = sdmom
        self.c1 = c1
        self.c2 = c2
        self.act0 = act0
        self.c3 = c3
        self.c4 = c4
        self.pmcor = pmcor
        self.cnst = cnst
        # Also works in parallel Asap:
        self.natoms = self.atoms.get_number_of_atoms() #GLOBAL number of atoms
        if len(self.atoms.constraints) == 1:
            # Process the FixAtoms constraint
            constr = self.atoms.constraints[0].index
            self.sdpos[constr] = 0.0
            self.sdmom[constr] = 0.0
            self.c1[constr] = 0.0
            self.c2[constr] = 0.0
            self.c3[constr] = 0.0
            self.c4[constr] = 0.0
            self.act0[constr] = 0.0
        if self._localfrict:
            self.asap_md.set_vector_constants(self.prefix+"act0", self.prefix+"c3",
                                              self.prefix+"c4", self.prefix+"pmcor",
                                              self.prefix+"cnst")
        else:
            self.asap_md.set_scalar_constants(self.act0, self.c3, self.c4,
                                              self.pmcor, self.cnst)

    def run(self, steps):
        assert(self.calculator is self.atoms.get_calculator())
        self.asap_md.run(steps, self.observers, self)

    def get_random(self, gaussian):
        return self.asap_md.get_random(gaussian)

    # Properties are not inherited, need to repeat them
    sdpos = property(lambda s: s.get("sdpos"), lambda s, x: s.set("sdpos", x))
    sdmom = property(lambda s: s.get("sdmom"), lambda s, x: s.set("sdmom", x))
    c1 = property(lambda s: s.get("c1"), lambda s, x: s.set("c1", x))
    c2 = property(lambda s: s.get("c2"), lambda s, x: s.set("c2", x))
    act0 = property(lambda s: s.get("act0"), lambda s, x: s.set("act0", x))
    c3 = property(lambda s: s.get("c3"), lambda s, x: s.set("c3", x))
    c4 = property(lambda s: s.get("c4"), lambda s, x: s.set("c4", x))
    pmcor = property(lambda s: s.get("pmcor"), lambda s, x: s.set("pmcor", x))
    cnst = property(lambda s: s.get("cnst"), lambda s, x: s.set("cnst", x))
    masses = property(lambda s: s.get("masses"), lambda s, x: s.set("masses", x))


def Langevin(atoms, *args, **kwargs):
    if (isinstance(atoms, ase.Atoms)
        and asap3.constraints.check_asap_constraints(atoms)
       ):
        # Nothing prevents Asap optimization
        sys.stderr.write("Using Asap-optimized C++-Langevin algorithm\n")
        return Langevin_Fast(atoms, *args, **kwargs)
    elif Langevin_ASE._lgv_version == 2:
        # ASE Langevin is version 2
        sys.stderr.write("Using ASE-based Langevin algorithm version 2\n")
        return ASE_Langevin_v2(atoms, *args, **kwargs)
    elif Langevin_ASE._lgv_version == 3:
        # ASE Langevin is version 3
        sys.stderr.write("Using ASE-based Langevin algorithm version 3\n")
        return ASE_Langevin_v3(atoms, *args, **kwargs)
    else:
        raise RuntimeError(
            "The Langevin dynamics in ASE has unsupported version {0}.".format(Langevin_ASE._lgv_version))



#!/usr/bin/env python
"""
Tests that EMT works with a twisted, left-hand unit cell when
minimimum-image convention is turned off.
"""
from __future__ import print_function

import sys
from numpy import *
from asap3 import *
from asap3.optimize.mdmin import MDMin
from asap3.analysis import CNA, CoordinationNumbers
from asap3.testtools import ReportTest
from ase.lattice.cubic import *


def testFCC(atoms, n, name):
    print("Test '%s': %d atoms" % (name, len(atoms)))
    ReportTest(("Number of atoms (%s)" % (name,)), len(atoms), n, 0)
    atoms.set_calculator(EMT(minimum_image=False))
    cn = CoordinationNumbers(atoms)
    ReportTest(("Coordination number is 12 (%s)" % (name,)),
               sum(equal(cn, 12)), len(atoms), 0)
    cna = CNA(atoms)
    ReportTest(("CNA says FCC (%s)" % (name,)),
               sum(equal(cna, 0)), len(atoms), 0)
    epot = atoms.get_potential_energy()/len(atoms)
    ReportTest(("Potential energy (%s)" % (name,)), epot, 0.0, 1e-3)

directions = [[5,1,3], [3,5,1], [2,2,-7]]
a = FaceCenteredCubic(directions=directions, size=(2,2,2), symbol="Cu")
print(a.get_cell())
testFCC(a, 5568, "FCC")


ReportTest.Summary()



import sys
import time
import numpy as np
import numbers

from asap3.Tools.ParameterOptimization.Optimization import ParameterOptimization

class ParameterSearch:
    def __init__(self, elements, calc, initparam, varparam, stdparam,
                 quantities, latticeconstants, calc_args=None, debug=False):
        self.elements = elements
        self.calc = calc
        self.initparam = initparam
        self.varparam = varparam
        self.stdparam = stdparam
        self.values = quantities
        self.latticeconstants = latticeconstants
        self.calc_args = calc_args
        self.debug = debug

    def get_new_parameters(self, parameters):
        new_parameters = {}
        for key in parameters.keys():
            std = self.stdparam[key]
            mask = np.array(self.varparam[key], float)
            change = np.random.uniform(1.0 - std, 1.0 + std, len(mask))
            change = mask * change - (mask - 1.0)
            new_parameters[key] = (np.array(parameters[key]) * change).tolist()
        return new_parameters

    def run(self, timer, type='follow', xtol=1e-1, ftol=5e-2, delta=5e-2,
            dat=None, txt=None):
        self.nsteps = 0
        self.numberlist = []
        self.errorlist = []
        self.fitparlist = []
        self.optparlist = []
        self.optvallist = []

        while True:
            self.nsteps += 1
            # Make a random set of parameters close to the
            if type == 'stay':
                if self.nsteps == 1:
                    fitpar = self.initparam
                else:
                    fitpar = self.get_new_parameters(self.initparam)
            elif type == 'follow':
                if self.nsteps == 1:
                    fitpar = self.initparam
                else:
                    fitpar = self.get_new_parameters(optpar)
            else:
                raise ValueError("Cannot determine the search type %s" % (type,))

            # Run optimization
            opt = ParameterOptimization(self.elements, self.calc, fitpar,
                                        self.varparam, self.values,
                                        self.latticeconstants, self.calc_args,
                                        debug=self.debug)
            (error, optpar, optval) = opt.fit(xtol, ftol, delta, txt)

            self.save(error, fitpar, optpar, optval)

            if isinstance(timer, numbers.Integral):
                if self.nsteps >= timer:
                    break
            else:
                if timer.done():
                    break

        self.write(dat)

    def save(self, error, fitpar, optpar, optval):
        # Sort and save the 10 best results
        length = len(self.errorlist)
        if length == 0:
            self.numberlist.append(self.nsteps)
            self.errorlist.append(error)
            self.fitparlist.append(fitpar)
            self.optparlist.append(optpar)
            self.optvallist.append(optval)
        else:
            new_min = -1
            new_rank = -1
            for i, err in enumerate(self.errorlist):
                op1 = np.empty(0)
                op2 = np.empty(0)
                for key in self.initparam.keys():
                    mask = np.array(self.varparam[key])
                    op1 = np.append(op1, np.array(optpar[key])[mask])
                    op2 = np.append(op2, np.array(self.optparlist[i][key])[mask])

                ov1 = np.empty(0)
                ov2 = np.empty(0)
                for v1, v2 in zip(optval, self.optvallist[i]):
                    ov1 = np.append(ov1, v1[3])
                    ov2 = np.append(ov2, v2[3])

                ov_dev_max = (2 * np.abs(ov1 - ov2) / np.abs(ov1 + ov2)).max()
                op_dev_max = (2 * np.abs(op1 - op2) / np.abs(op1 + op2)).max()

                if ov_dev_max < 2e-2 and op_dev_max < 5e-2 and new_min < 0 and i > 0:
                    new_min = i

                if error < err and new_rank < 0 and i > 0:
                    new_rank = i

            #print new_min, new_rank, length
            if new_min < 0:
                # New minimum found
                if new_rank > 0:
                    # Better error function
                    self.numberlist.insert(new_rank, self.nsteps)
                    self.errorlist.insert(new_rank, error)
                    self.fitparlist.insert(new_rank, fitpar)
                    self.optparlist.insert(new_rank, optpar)
                    self.optvallist.insert(new_rank, optval)
                    if length >= 21:
                        del self.numberlist[-1]
                        del self.errorlist[-1]
                        del self.fitparlist[-1]
                        del self.optparlist[-1]
                        del self.optvallist[-1]
                else:
                    # Worst error function
                    if length < 21:
                        self.numberlist.append(self.nsteps)
                        self.errorlist.append(error)
                        self.fitparlist.append(fitpar)
                        self.optparlist.append(optpar)
                        self.optvallist.append(optval)
            else:
                # Old minimum found
                if  new_rank > 0 and new_rank <= new_min:
                    # Better error function
                    del self.numberlist[new_min]
                    del self.errorlist[new_min]
                    del self.fitparlist[new_min]
                    del self.optparlist[new_min]
                    del self.optvallist[new_min]
                    self.numberlist.insert(new_rank, self.nsteps)
                    self.errorlist.insert(new_rank, error)
                    self.fitparlist.insert(new_rank, fitpar)
                    self.optparlist.insert(new_rank, optpar)
                    self.optvallist.insert(new_rank, optval)

    def write(self, txt, name="search"):
        if txt is None:
            f = sys.stdout
        else:
            f = open(txt, 'a')
        T = time.localtime()
        f.write("Parameter %s %02i-%02i-%i " % (name, T[2], T[1], T[0]) +
                "at %02i:%02i:%02i\n" % (T[3], T[4], T[5]) +
                "-" * 90 + "\n" +
                "%i optimizations has been made. " % (self.nsteps,) +
                "Below the initial and the 20 best are listed:\n\n")

        for num, error, fitpar, optpar, optval in zip(self.numberlist, self.errorlist,
                                                      self.fitparlist, self.optparlist,
                                                      self.optvallist):
            f.write("Optimization no. %i with error function %.3f:\n" % (num, error))
            for key in fitpar.keys():
                f.write("  %s%s parameters (Initial, Optimal, Variable):\n" % (key[0], key[1]))
                for pi, po, v in zip(fitpar[key], optpar[key], self.varparam[key]):
                    f.write("  %12.9f  %12.9f  %s\n" % (pi, po, v))
            f.write("\n  Fitting values (Quantity, Structure: " +
                    "Target, Optimal, Weight, Deviation):\n")
            devsum = 0
            for ival, oval in zip(self.values, optval):
                if len(oval[2]) > 1:
                    struct = "%s%s-%s" % (oval[2] + (oval[1],))
                else:
                    struct = "%s-%s" % (oval[2] + (oval[1],))
                name = "%s, %s:" % (oval[0], struct)
                dev = np.abs(oval[3] / ival[3] - 1) * 100
                devsum += dev
                f.write("  %-45s  %7.3f  " % (name, ival[3]) +
                        "%10.6f  %5.3f  %7.2f%%\n" % (oval[3], ival[4], dev))
            f.write("%84.2f%%\n\n" % (devsum / len(self.values),))
        f.close()





"""Test force calculations alone with the purpose of optimizing threading."""
from __future__ import print_function

from asap3 import *
from asap3.timing import report_timing
from ase.lattice.cubic import FaceCenteredCubic
import time

nsteps = 10

start = time.time()
if len(sys.argv) >= 2 and sys.argv[1] == '-t':
    AsapThreads()   
if len(sys.argv) >= 2 and sys.argv[1] == '-T':
    AsapThreads(6)   

atoms = FaceCenteredCubic(size=(100,100,50), symbol='Cu')
atoms.set_calculator(EMT())
print("Number of atoms:", len(atoms))

d = 0.1
pos = atoms.arrays['positions']  # Nasty!

for i in range(nsteps):
    pos[50][0] += d
    d = -d
    f = atoms.get_forces()
    
report_timing()

print("Wall time elapsed:", time.time() - start)





import resource
from asap3.mpi import world
import sys

if sys.platform == 'darwin':
    _memory_unit = 1   # Bytes
else:
    _memory_unit = 1024  # kB

_megabytes = 1024.0**2 / _memory_unit

def get_max_memory():
    """Return the max resident memory used by the process.

    It tries to return the value in MB, but this may fail on
    some platforms.
    """
    mem = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
    if world.size > 1:
        mem = world.sum(mem)
    return mem / _megabytes




from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest

print("Making atoms")
atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                          size=(25,25,25), symbol="Cu")
memory_usage(atoms)

print("Attaching EMT potential")
atoms.set_calculator(EMT())
memory_usage(atoms)

print("Calculating forces")
atoms.get_forces()
memory_usage(atoms)

print("Test passed!")



from __future__ import print_function
from asap3 import EMT2013
from asap3.EMT2013Parameters import sihb_PtY_parameters
from ase.lattice import  bulk 
from asap3.testtools import ReportTest
from ase.utils import basestring

def make_Pt():
    return bulk('Pt', 'fcc', orthorhombic=True).repeat((6,6,6))

def make_Y():
        return bulk('Y', 'hcp', orthorhombic=True).repeat((6,6,6))

atoms = make_Pt()
atoms.set_calculator(EMT2013(sihb_PtY_parameters, True))
e_Pt1 = atoms.get_potential_energy()
print("Pt (1):", e_Pt1)

atoms = make_Y()
atoms.set_calculator(EMT2013(sihb_PtY_parameters, True))
e_Y1 = atoms.get_potential_energy()
print("Y (1):", e_Y1)

calc = EMT2013(sihb_PtY_parameters)

atoms = make_Pt()
atoms.set_calculator(calc)
e_Pt2 = atoms.get_potential_energy()
print("Pt (1):", e_Pt2)

atoms = make_Y()
atoms.set_calculator(calc)
e_Y2 = atoms.get_potential_energy()
print("Y (1):", e_Y2)

ReportTest("Energy of first element (Pt)", e_Pt2, e_Pt1, 1e-9)
ReportTest("Energy of second element (Y)", e_Y2, e_Y1, 1e-9)
ReportTest.Summary()



from numpy import *
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from ase.constraints import Filter
from asap3.testtools import *

atoms = FaceCenteredCubic(size=(5, 5, 5), symbol="Cu")
#p = RasMol(atoms)

ReportTest("Total number of atoms", len(atoms), 500, 0)

ss = Filter(atoms, indices=[2,5,7,10,25,100])
ReportTest("number of atoms in subset",len(ss), 6, 0)

a = atoms[25]
b = ss[4]
assert a.index == b.index
assert a.atoms is b.atoms

a2 = Atoms(ss)
ReportTest("number of atoms in copy of subset",len(a2), 6, 0)

a = Filter(atoms, indices=(5,6,7,8))
ReportTest("Number of atoms in subset", len(a), 4, 0)

a.set_atomic_numbers(47*ones(4, int32))
ReportTest("subset: z[0]", a.get_atomic_numbers()[0], 47, 0)
ReportTest("subset: z[1]", a.get_atomic_numbers()[1], 47, 0)
ReportTest("subset: z[2]", a.get_atomic_numbers()[2], 47, 0)
ReportTest("subset: z[3]", a.get_atomic_numbers()[3], 47, 0)
z = atoms.get_atomic_numbers()
ReportTest("full: z[0]", z[0], 29, 0)
ReportTest("full: z[4]", z[4], 29, 0)
ReportTest("full: z[5]", z[5], 47, 0)
ReportTest("full: z[8]", z[8], 47, 0)
ReportTest("full: z[9]", z[9], 29, 0)
ReportTest("full: z[100]", z[100], 29, 0)

r = a.get_positions()
r[2] = array([10.0, 20.0, 30.0])
a.set_positions(r)

r = atoms.get_positions()
ReportTest("full: r[7,0]", r[7,0], 10.0, 0)
ReportTest("full: r[7,1]", r[7,1], 20.0, 0)
ReportTest("full: r[7,2]", r[7,2], 30.0, 0)

ra = atoms[7].position
rb = a[2].position
ReportTest("x coordinate of number 2/7", ra[0], rb[0], 0)
ReportTest("y coordinate of number 2/7", ra[1], rb[1], 0)
ReportTest("z coordinate of number 2/7", ra[2], rb[2], 0)

#print a.get_positions()

ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from numpy import *
from asap3.testtools import ReportTest
from ase.lattice.cubic import *
from ase.lattice.compounds import *
from OpenKIM_modelname import openkimmodel
from asap3.EMT2013Parameters import sihb_PtY_parameters

element = 'Pt'
element2 = ('Y', 'Pt')
lc2 = 4.125
sizes = arange(10,0,-1)

energy = None
force = None


for s in sizes:
    atoms = FaceCenteredCubic(symbol=element, size=(s,s,s), pbc=True)
    r = atoms.get_positions()
    for i in range(0, len(r), 4):
        r[i][0] += 0.1
    atoms.set_positions(r)
    atoms.set_calculator(EMT2013(sihb_PtY_parameters))
    e = atoms.get_potential_energy() / len(atoms)   
    print("%5i atoms: E = %.5f eV/atom" % (len(atoms), e))
    if energy is None:
        energy = e
    else:
        ReportTest("Energy for size %i (%i atoms)" % (s, len(atoms)),
                   e, energy, 1e-8)
    f = atoms.get_forces()
    if force is None:
        force = f[:4]
    else:
        for i in range(len(f)):
            for j in range(3):
                ReportTest("Force for size %i atom %i of %i component %i"
                           % (s, i, len(atoms), j),
                           f[i,j], force[i % 4, j], 1e-8, silent=True)
        #print force
        #print f[:10]
    del atoms


energy = None
force = None

for s in sizes:
    atoms = L1_2(symbol=element2, size=(s,s,s), latticeconstant=lc2, pbc=True)
    r = atoms.get_positions()
    for i in range(0, len(r), 4):
        r[i][0] += 0.1
    atoms.set_positions(r)
    atoms.set_calculator(EMT2013(sihb_PtY_parameters))
    e = atoms.get_potential_energy() / len(atoms)   
    print("%5i atoms: E = %.5f eV/atom" % (len(atoms), e))
    if energy is None:
        energy = e
    else:
        ReportTest("Energy for size %i (%i atoms)" % (s, len(atoms)),
                   e, energy, 1e-8)
    f = atoms.get_forces()
    if force is None:
        force = f[:4]
    else:
        for i in range(len(f)):
            for j in range(3):
                ReportTest("Force for size %i atom %i of %i component %i"
                           % (s, i, len(atoms), j),
                           f[i,j], force[i % 4, j], 1e-8, silent=True)
        #print force
        #print f[:10]
    del atoms


ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from asap3.constraints import FixAtoms
from ase.lattice.cubic import FaceCenteredCubic
from ase.io import read, write
from asap3.io import Trajectory
import os
import numpy as np

usemask = True

fn = "testconstraint"
atoms = FaceCenteredCubic(symbol='Cu', size=(3,3,3))
if usemask:
    c = FixAtoms(mask=np.arange(len(atoms))<3)
else:
    c = FixAtoms(indices=(2,3))
atoms.set_constraint(c)
print(c)

traj = Trajectory(fn+'.trj', "w", atoms)
traj.write()
traj.close()
del traj

traj = BundleTrajectory(fn+'.bundle', "w", atoms)
traj.write()
traj.close()
del traj

traj = Trajectory(fn+'.traj', "w", atoms)
traj.write()
traj.close()
del traj

traj2 = Trajectory(fn+'.trj')
atoms2 = traj2[-1]
c2 = atoms2.constraints[0]

print("*** Trajectory")
print("Original class:", c.__class__)
print("New class:", c2.__class__)
assert c.__class__ is c2.__class__

traj2 = read(fn+'.bundle')
c2 = atoms2.constraints[0]

print("*** BundleTrajectory")
print("Original class:", c.__class__)
print("New class:", c2.__class__)
assert c.__class__ is c2.__class__

print("*** Cleaning up")
os.unlink(fn+'.traj')
os.unlink(fn+'.trj')
BundleTrajectory.delete_bundle(fn+'.bundle')

print("Test passed.")




# Copyright (C) 2004-2011 Jakob Schiotz and Center for Individual
# Nanoparticle Functionality, Department of Physics, Technical
# University of Denmark.  Email: schiotz@fysik.dtu.dk
#
# This file is part of Asap version 3.
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# version 3 as published by the Free Software Foundation.  Permission
# to use other versions of the GNU Lesser General Public License may
# granted by Jakob Schiotz or the head of department of the
# Department of Physics, Technical University of Denmark, as
# described in section 14 of the GNU General Public License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# and the GNU Lesser Public License along with this program.  If not,
# see <http://www.gnu.org/licenses/>.

"""The Asap module.

The following **classes** are defined in this module:

`ListOfAtoms`: A list of atoms object used for the simulations.

`ParallelAtoms`: A parallel version of `ListOfAtoms`.  Only defined if
parallel simulations are possible.  Should not be created directly,
use `MakeParallelAtoms`.

`EMT`: The Effective Medium Theory potential.

`MonteCarloEMT`: EMT potential with optimizations for Monte Carlo simulations.

`MoPotential`: An experimental molybdenum potential.

`LennardJones`: Calculator using Lennard-Jones potential.

`Morse`: Calculator using Morse potential.

`BrennerPotential`: Calculates the Brenner potential.

The following **functions** are defined:

`Verbose`: Changes the verbosity level of the C code.

`CNA`: Runs Common Neighbor Analysis on a (Parallel)ListOfAtoms object.

`CoordinationNumber`: Calculates coordination numbers for the atoms
in a (Parallel)ListOfAtoms object.

`MakeParallelAtoms`: A factory function creating a ParallelAtoms
object safely.  Only defined if parallel simulations are possible.

This module detects if parallel simulations are possible, and then
loads the serial or the parallel version of the C module into memory.
If parallel simulations are possible, a Python exit function is set,
so MPI is shut down if one of the processes exits.

*Note on the automatic documentation*: Most of the classes mentioned
above are not listed below.  This is because the way epydoc_ works
interacts badly with the structure of these modules.  Click on
class/function name above to see the documentation.

.. _epydoc: http://epydoc.sf.net
"""


__docformat__ = "restructuredtext en"

import sys, os
from asap3.version import __version__
from asap3.Internal.Builtins import _asap, parallelpossible, AsapError, set_verbose
from asap3.Internal.UtilityFunctions import print_version, get_version, \
     get_short_version, DebugOutput, memory_usage, print_memory
from asap3.Internal.BuiltinPotentials import EMT, MonteCarloEMT, \
     EMTParameters, EMTDefaultParameters, EMTRasmussenParameters, \
     LennardJones, BrennerPotential, Morse, EMT2013, EMT2011, RGL, Gupta, \
     RahmanStillingerLemberg, MetalOxideInterface, MetalOxideInterface2
from asap3.Internal.EMTParameters import EMTStandardParameters, \
     EMThcpParameters, EMTMetalGlassParameters
from asap3.Internal.Threads import AsapThreads
from asap3.Internal.MonteCarloAtoms import MonteCarloAtoms
from asap3.Internal.checkversion import check_version
from asap3.io.trajectory import Trajectory
from asap3.io.bundletrajectory import BundleTrajectory
from asap3.md.verlet import VelocityVerlet
from asap3.md.langevin import Langevin
from asap3.md import MDLogger
    
if parallelpossible:
    from asap3.Internal.ParallelListOfAtoms import ParallelAtoms, \
        MakeParallelAtoms
    from asap3.Internal.Collector import Collector
    from ase.parallel import paropen

# OpenKIM may or may not be built into Asap
from asap3.Internal.BuiltinPotentials import OpenKIMsupported
if OpenKIMsupported:
    from asap3.Internal.OpenKIMcalculator import OpenKIMcalculator, OpenKIMavailable

import ase
from ase import Atoms
from ase.visualize import view
import ase.units as units
# from ase.md.velocitydistribution import MaxwellBoltzmannDistribution, Stationary

try:
    from ase.calculators.calculator import PropertyNotImplementedError
except ImportError:
    # ASE version 3.12 or earlier.
    class PropertyNotImplementedError(NotImplementedError):
        pass

NeighborList = _asap.NeighborList
NeighborCellLocator = _asap.NeighborCellLocator
FullNeighborList = _asap.FullNeighborList
heap_mallinfo = _asap.heap_mallinfo

# Check Asap installation for consistency
check_version()

# Check that a modern ASE is used in parallel simulations
#
# Due to a change in ASEs build system, older ASEs will not detect
# that ASAP is running in parallel, and ase.parallel will not work.
if parallelpossible:
    import ase.parallel
    import asap3.mpi
    if asap3.mpi.world.size != ase.parallel.world.size:
        raise ImportError("Asap requires ASE version 3.12 or updated developer installation.")
    



#! /usr/bin/env python
#PBS -N TinyLangevin
#PBS -q long
#PBS -l nodes=1:opteron4

"""testLangevin.py - test the Langevin dynamics.

Usage: python testLangevin.py

Tests Langevin dynamics using the EMT Copper potential.
"""
from __future__ import print_function

import sys, time
from numpy import *
import asap3
from asap3.testtools import ReportTest
from asap3.md.langevin import Langevin
from asap3.md.verlet import VelocityVerlet
from ase.lattice.cubic import FaceCenteredCubic
from ase import units
from asap3.io import Trajectory

nequil = 10000
nequilprint = 25
nsteps = 500000
nprint = 250
tolerance = 0.01
nminor = 25
timestep = 5 * units.fs

# Use Langevin from ASE if --ase argument is given
if len(sys.argv) > 1 and sys.argv[1] == "--ase":
    print("--ase specified: Using unmodified ASE Langevin dynamics.")
    from ase.md.langevin import Langevin

# Set up atoms in a regular simple-cubic lattice.
atoms = FaceCenteredCubic(size=(1,1,2), symbol="Cu", pbc=False,
                        latticeconstant = 3.5)
atoms.set_calculator(asap3.EMT())
    
ReportTest("Number of atoms", len(atoms), 8, 0)

# Make a small perturbation of the momenta
atoms.set_momenta(1e-6 * random.random([len(atoms), 3]))
print("Initializing ...")
predyn = VelocityVerlet(atoms, 0.5)
predyn.run(2500)

initr = atoms.get_positions()
initp = atoms.get_momenta()


def targetfunc(params, x):
    return params[0] * exp(-params[1] * x) + params[2]

output =open("Langevin.dat", "w")

results = []

for temp, frict, fixcm in ((0.025, 0.001, True), (0.025, 0.0001, False),):
    dyn = Langevin(atoms, timestep, temp, frict, fixcm=fixcm)
    traj = Trajectory("Langevin-{0}-{1}-{2}.traj".format(temp, frict, fixcm),
                      "w", atoms)
    print("")
    print("Testing Langevin dynamics with T = %f eV and lambda = %f" % (temp, frict))
    ekin = atoms.get_kinetic_energy()/len(atoms)
    print(ekin)
    output.write("%.8f\n" % ekin)
    temperatures = [(0, 2.0 / 3.0 * ekin)]
    a = 0.1
    b = frict
    c = temp
    print("Equilibrating ...")
    tstart = time.time()
    for i in xrange(1,nequil+1):
        dyn.run(nminor)
        ekin = atoms.get_kinetic_energy() / len(atoms)
        if i % nequilprint == 0:

            print("%.6f  T = %.6f (goal: %f)" % \
                  (ekin, 2.0/3.0 * ekin, temp))
        output.write("%.8f\n" % ekin)
    tequil = time.time() - tstart
    print("This took %s minutes." % (tequil / 60))
    output.write("&\n")
    temperatures = []
    print("Taking data")
    tstart = time.time()
    for i in xrange(1,nsteps+1):
        dyn.run(nminor)
        ekin = atoms.get_kinetic_energy() / len(atoms)
        temperatures.append(2.0/3.0 * ekin)
        if i % nprint == 0:
            tnow = time.time() - tstart
            tleft = (nsteps-i) * tnow / i
            print("%.6f    (time left: %.1f minutes)" % (ekin, tleft/60))
            traj.write()
        output.write("%.8f\n" % ekin)
    output.write("&\n")
    temperatures = array(temperatures)
    mean = sum(temperatures) / len(temperatures)
    print("Mean temperature:", mean, "eV")
    print("")
    print("This test is statistical, and may in rare cases fail due to a")
    print("statistical fluctuation.")
    print("")
    expected = temp
    if fixcm:
        expected *= 7.0 / 8.0
    results.append((mean, (mean - expected)/expected))
    ReportTest("Mean temperature:", mean, expected, tolerance*temp)

print("RESULTS:")
for r in results:
    print("%.8f  %.8f" % r)
    
output.close()

